<?php

namespace Teknisa\DF\Factories;

use Doctrine\ORM\EntityManager;

use Teknisa\Libs\Util\DateUtil;
use Geral\Entities\Itemproduto;
use Geral\Entities\Itemencargo;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class NotafiscalLancamentoFactory {

    const ESTOQUE_SEPARADOR = ' ';

    const PROCEDURE_DELETA_NF = 'DELETA_NF';

    private $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    // Formata a data de forma que possa ser inserida no banco
    public static function formataData($data) {
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);

        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);
        return $data;
    }

    public function procedureDeletaNf($filial, $lancamento, $corpoItem = null){
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_DELETA_NF);
            $procedure->addParam(new Param('P_FILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_LANCTONF', Param::PARAM_INPUT, $lancamento, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDCORPITEM', Param::PARAM_INPUT, $corpoItem, Param::PARAM_TYPE_STR));
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
        public function existeBoleto($filial, $nrbolpag){
             $sql = "SELECT 1 
                       FROM BOLETPAGAR 
                      WHERE CDFILIAL    = :P_CDFILIAL
                        AND NRLANBOLPAG = :P_NRLANBOLPAG";
             
        $params = array("P_CDFILIAL" => $filial,
                        "P_NRLANBOLPAG" => $nrbolpag);
             
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return sizeof($std->fetchAll()) > 0 ? "S" : "N"; 
    }
    
    public function verificaRegistros($cdfilial, $lancamento) {
       
        $sql = "SELECT
                    COUNT(*) AS QTDREGISTRO
                FROM
                    INFOCOMPRANFE
                WHERE
                    CDFILIAL = :P_CDFILIAL
                    AND NRLANCTONF = :P_NRLANCTONF";
                    
        $params = array(
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTONF" => $lancamento
        );
                    
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch()["QTDREGISTRO"];
        
    }
    
    public function atualizaInfoCompra($filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg){
        $qtdRegistros = $this->verificaRegistros($filial, $lancamento);
        if($qtdRegistros > 0){
            $informacoes = $this->getInfoNota($filial, $lancamento);
            $this->alteraInfoCompra($informacoes,$filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg);
        }
    }
    
    public function alteraInfoCompra($informacoes,$filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg){
        
        $idAtivo = $informacoes[0]['IDATIVO'];
        $dtInclusao = $informacoes[0]['DTINCLUSAO'];
        $dtUltatu = $informacoes[0]['DTULTATU'];
        $nrorgInclusao = $informacoes[0]['NRORGINCLUSAO'];
        $cdoperinclusao = $informacoes[0]['CDOPERINCLUSAO'];
        $nrorgUltatu = $informacoes[0]['NRORGULTATU'];
        $cdoperUltatu = $informacoes[0]['CDOPERULTATU'];
        
       
        //$connection = $this->entityManager->getConnection();
        //$connection->beginTransaction();
        
         $sql = "UPDATE INFOCOMPRANFE 
                    SET 
                    DSNOTAEMPENHO = :DSNOTAEMPENHO,
                    CDPEDIDO = :CDPEDIDO,
                    CDCONTRATO = :CDCONTRATO,
                    IDATIVO = :IDATIVO,
                    DTINCLUSAO = :DTINCLUSAO,
                    DTULTATU = :DTULTATU,
                    NRORGINCLUSAO = :NRORGINCLUSAO,
                    CDOPERINCLUSAO = :CDOPERINCLUSAO,
                    NRORGULTATU = :NRORGULTATU,
                    CDOPERULTATU = :CDOPERULTATU,
                    NRORG = :NRORG
                    WHERE  CDFILIAL = :CDFILIAL
                        AND NRLANCTONF = :NRLANCTONF";
        
        $params = array(
            "CDFILIAL"        =>  $filial,
            "NRLANCTONF"      =>  $lancamento,
            "DSNOTAEMPENHO"   =>  $notaEmp,
            "CDPEDIDO"        =>  $pedido,
            "CDCONTRATO"      =>  $contrato,
            "IDATIVO"         =>  $idAtivo,
            "DTINCLUSAO"      =>  $dtInclusao,
            "DTULTATU"        =>  $dtUltatu,
            "NRORGINCLUSAO"   =>  $nrorgInclusao,
            "CDOPERINCLUSAO"  =>  $cdoperinclusao,
            "NRORGULTATU"     =>  $nrorgUltatu,
            "CDOPERULTATU"    =>  $cdoperUltatu,
            "NRORG"           =>  $nrorg 
        );
        
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
        //$connection->commit();
        
        //var_dump($informacoes, $lancamento,$filial,$notaEmp,$pedido,$contrato);
    }
    
     public function getInfoNota($filial, $lancamento) {
         $sql = "SELECT IDATIVO,
                        DTINCLUSAO,
                        DTULTATU,
                        NRORGINCLUSAO,
                        CDOPERINCLUSAO,
                        NRORGULTATU,
                        CDOPERULTATU
                FROM NOTAFISCAL
                WHERE CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF";
                
        $params = array(
            "P_NRLANCTONF"        => $lancamento,
            "P_CDFILIAL"          => $filial
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
         
    }
    
    public function salvaInfoCompra($filial, $lancamento, $notaEmp,$pedido,$contrato,$nrorg){
        
        $informacoes = $this->getInfoNota($filial, $lancamento);
        //$this->verificaRegistros($filial, $lancamento);
        $idAtivo = $informacoes[0]['IDATIVO'];
        $dtInclusao = $informacoes[0]['DTINCLUSAO'];
        $dtUltatu = $informacoes[0]['DTULTATU'];
        $nrorgInclusao = $informacoes[0]['NRORGINCLUSAO'];
        $cdoperinclusao = $informacoes[0]['CDOPERINCLUSAO'];
        $nrorgUltatu = $informacoes[0]['NRORGULTATU'];
        $cdoperUltatu = $informacoes[0]['CDOPERULTATU'];
        
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
         $sql = "INSERT INTO INFOCOMPRANFE (
                CDFILIAL,
                NRLANCTONF,
                DSNOTAEMPENHO,
                CDPEDIDO,
                CDCONTRATO,
                IDATIVO,
                DTINCLUSAO,
                DTULTATU,
                NRORGINCLUSAO,
                CDOPERINCLUSAO,
                NRORGULTATU,
                CDOPERULTATU,
                NRORG
            ) VALUES (
                :P_CDFILIAL,
                :P_NRLANCTONF,
                :P_DSNOTAEMPENHO,
                :P_CDPEDIDO,
                :P_CDCONTRATO,
                :P_IDATIVO,
                :P_DTINCLUSAO,
                :P_DTULTATU,
                :P_NRORGINCLUSAO,
                :P_CDOPERINCLUSAO,
                :P_NRORGULTATU,
                :P_CDOPERULTATU,
                :P_NRORG
            )";
            
        $params = array(
            "P_CDFILIAL"        =>  $filial,
            "P_NRLANCTONF"      =>  $lancamento,
            "P_DSNOTAEMPENHO"   =>  $notaEmp,
            "P_CDPEDIDO"        =>  $pedido,
            "P_CDCONTRATO"      =>  $contrato,
            "P_IDATIVO"         =>  $idAtivo,
            "P_DTINCLUSAO"      =>  $dtInclusao,
            "P_DTULTATU"        =>  $dtUltatu,
            "P_NRORGINCLUSAO"   =>  $nrorgInclusao,
            "P_CDOPERINCLUSAO"  =>  $cdoperinclusao,
            "P_NRORGULTATU"     =>  $nrorgUltatu,
            "P_CDOPERULTATU"    =>  $cdoperUltatu,
            "P_NRORG"           =>  $nrorg 
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $connection->commit();
        
    }
    
    public function novoItemproduto(
        $filial, $seqLancamento, $nrseqitprod, $cdproduto, $cdprodemis,
        $dsprodemis, $qtitemprod, $vrbrutitem, $vrunititem, $vrdescitem,
        $vrtotaitem, $organizacao, $operador, $nrautoforn = null, $cdfilimovi = null,
        $cdmarca = null, $dtprogentaf = null, $cdfisopeitem = null, $cdalmoxarife = self::ESTOQUE_SEPARADOR,
        $cdlocalestoq = self::ESTOQUE_SEPARADOR, $nrloteestq = self::ESTOQUE_SEPARADOR,
        $nrsublote = self::ESTOQUE_SEPARADOR, $idatuvrestoq = 'S',
        $vrcontitem = 0, $idconforitem = 'S'
    ) {
        try {
  
            $data = DateUtil::getDataAtual();
            
            switch(substr($cdfisopeitem, 0, 1)) {
                case '5': 
                    $cdfisopeitem = '1'.substr($cdfisopeitem, 1, strlen($cdfisopeitem) - 1);
                    break;
                case '6': 
                    $cdfisopeitem = '2'.substr($cdfisopeitem, 1, strlen($cdfisopeitem) - 1);
                    break;
                case '7': 
                    $cdfisopeitem = '3'.substr($cdfisopeitem, 1, strlen($cdfisopeitem) - 1);
                    break;
            }
 
            $itemproduto = new Itemproduto();
            $itemproduto->setCdfilial($filial);
            $itemproduto->setNrlanctonf($seqLancamento);
            $itemproduto->setNrseqitprod($nrseqitprod);
            $itemproduto->setCdproduto($cdproduto);
            $itemproduto->setCdprodemis($cdprodemis);
            $itemproduto->setDsprodemis($dsprodemis);
            $itemproduto->setQtitemprod($qtitemprod);
            $itemproduto->setVrbrutitem($vrbrutitem);
            $itemproduto->setVrunititem($vrunititem);
            $itemproduto->setVrdescitem($vrdescitem);
            $itemproduto->setVrtotaitem($vrtotaitem);
            $itemproduto->setCdfilimovi($cdfilimovi);
            $itemproduto->setCdmarca($cdmarca);
            $itemproduto->setCdfisopeitem($cdfisopeitem);
            $itemproduto->setCdalmoxarife($cdalmoxarife);
            $itemproduto->setCdlocalestoq($cdlocalestoq);
            $itemproduto->setNrloteestq($nrloteestq);
            $itemproduto->setNrsublote($nrsublote);
            $itemproduto->setIdatuvrestoq($idatuvrestoq);
            $itemproduto->setVrcontitem($vrcontitem);
            $itemproduto->setIdconforitem($idconforitem);
            $itemproduto->setNrautoforn($nrautoforn);

            if($dtprogentaf) {
                $dtProg = new \DateTime(str_replace("/", "-", $dtprogentaf));
                $itemproduto->setDtprogentaf($dtProg);
            }

            $itemproduto->setNrorg($organizacao);
            $itemproduto->setNrorginclusao($organizacao);
            $itemproduto->setNrorgultatu($organizacao);
            $itemproduto->setCdoperinclusao($operador);
            $itemproduto->setCdoperultatu($operador);
            $itemproduto->setDtinclusao($data);
            $itemproduto->setDtultatu($data);

            $itemproduto->persist();

            return $itemproduto;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function editarItemproduto(
        Itemproduto $itemproduto, $cdprodemis, $dsprodemis, $qtitemprod,
        $vrbrutitem, $vrunititem, $vrdescitem, $vrtotaitem, $organizacao,
        $operador, $nrautoforn = null, $cdfilimovi = null, $cdmarca = null,
        $cdalmoxarife = self::ESTOQUE_SEPARADOR, $cdlocalestoq = self::ESTOQUE_SEPARADOR,
        $nrloteestq = self::ESTOQUE_SEPARADOR, $nrsublote = self::ESTOQUE_SEPARADOR,
        $idatuvrestoq = 'S', $vrcontitem = 0, $idconforitem = 'S'
    ) {
        try {
            $data = DateUtil::getDataAtual();

            $itemproduto->setCdprodemis($cdprodemis);
            $itemproduto->setDsprodemis($dsprodemis);
            $itemproduto->setQtitemprod($qtitemprod);
            $itemproduto->setVrbrutitem($vrbrutitem);
            $itemproduto->setVrunititem($vrunititem);
            $itemproduto->setVrdescitem($vrdescitem);
            $itemproduto->setVrtotaitem($vrtotaitem);
            $itemproduto->setCdfilimovi($cdfilimovi);
            $itemproduto->setCdmarca($cdmarca);
            $itemproduto->setCdalmoxarife($cdalmoxarife);
            $itemproduto->setCdlocalestoq($cdlocalestoq);
            $itemproduto->setNrloteestq($nrloteestq);
            $itemproduto->setNrsublote($nrsublote);
            $itemproduto->setIdatuvrestoq($idatuvrestoq);
            $itemproduto->setVrcontitem($vrcontitem);
            $itemproduto->setIdconforitem($idconforitem);
            $itemproduto->setNrautoforn($nrautoforn);

            $itemproduto->setNrorgultatu($organizacao);
            $itemproduto->setCdoperultatu($operador);
            $itemproduto->setDtultatu($data);

            $itemproduto->persist();

            return $itemproduto;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function novoEncargo(
        $filial, $lancamento, $cdencargo, $vritemenca, $organizacao,
        $operador, $vrenmoedestr = null, $idalterado = 'N', $idativo = 'S'
    ) {
        try {
            $data = DateUtil::getDataAtual();

            $itemencargo = new Itemencargo();
            $itemencargo->setCdfilial($filial);
            $itemencargo->setNrlanctonf($lancamento);
            $itemencargo->setCdencargo($cdencargo);
            $itemencargo->setVritemenca($vritemenca);
            $itemencargo->setVrenmoedestr($vrenmoedestr);
            //$itemencargo->setIdalterado($idalterado);

            $itemencargo->setNrorg($organizacao);
            $itemencargo->setNrorginclusao($organizacao);
            $itemencargo->setNrorgultatu($organizacao);
            $itemencargo->setCdoperinclusao($operador);
            $itemencargo->setCdoperultatu($operador);
            $itemencargo->setDtinclusao($data);
            $itemencargo->setDtultatu($data);
            $itemencargo->persist();
            //$itemencargo->setIdativo($idativo);

            return $itemencargo;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function editarEncargo(
        Itemencargo $itemencargo, $vritemenca, $organizacao, $operador, $vrenmoedestr = null, $idalterado = 'N'
    ) {
        try {
            $data = DateUtil::getDataAtual();

            $itemencargo->setVritemenca($vritemenca);
            $itemencargo->setVrenmoedestr($vrenmoedestr);
            //$itemencargo->setIdalterado($idalterado);

            $itemencargo->setNrorgultatu($organizacao);
            $itemencargo->setCdoperultatu($operador);
            $itemencargo->setDtultatu($data);
            //$itemencargo->setIdativo($idativo);
            $itemencargo->persist();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
     public function buscaItensRateionf($filial, $lancamento){
         $sql = "SELECT DISTINCT I.CDPRODUTO,
                  P.NMPRODUTO,
                  I.NRSEQITPROD,
                  ROUND(I.VRCONTITEM, 2) AS VRCONTITEM,
                  P.CDARVPROD,
                  N.CDFILIRECE,
                  N.NRLANCTONF,
                  N.CDFILIAL
                FROM ITEMPRODUTO I,
                  PRODUTO P,
                  RATEIONF R,
                  NOTAFISCAL N
                WHERE (N.CDFILIAL    = :P_CDFILIAL)
                AND (N.NRLANCTONF    = :P_NRLANCTONF)
                AND (I.CDFILIAL      = :P_CDFILIAL)
                AND (I.NRLANCTONF    = :P_NRLANCTONF)
                AND (R.CDFILIAL(+)   = :P_CDFILIAL)
                AND (R.NRLANCTONF(+) = :P_NRLANCTONF)
                AND (I.NRSEQITPROD   = R.NRSEQITPROD(+))
                AND (I.CDPRODUTO     = P.CDPRODUTO)
                ORDER BY I.NRSEQITPROD";
             
        $params = array("P_CDFILIAL" => $filial,
                        "P_NRLANCTONF" => $lancamento);
             
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
    }
    
    public function buscaRateionf($filial, $lancamento, $arrayItens){
         $sql = "SELECT R.CDFILIAL, R.NRLANCTONF, R.NRSEQITPROD, R.CDFILICCUST, F.NMFILIAL,
                       R.CDCENTCUSNF, C.NMCENTCUST, ROUND(R.VRRATENF, 2) AS VRRATENF, DSRATEIO, VRPERRAT
                FROM RATEIONF R, FILIAL F, CENTCUST C
                WHERE (R.CDFILIAL    = :P_CDFILIAL)    AND
                      (R.NRLANCTONF  = :P_NRLANCTONF)  AND
                      (INSTR(NVL(:P_NRSEQITPROD, R.NRSEQITPROD), R.NRSEQITPROD)        > 0) AND
                      (R.CDFILICCUST = F.CDFILIAL)     AND
                      (R.CDCENTCUSNF = C.CDCENTCUST)
                ORDER BY R.CDFILICCUST, R.CDCENTCUSNF";
             
        $params = array("P_CDFILIAL" => $filial,
                        "P_NRLANCTONF" => $lancamento,
                        "P_NRSEQITPROD" => $arrayItens);
             
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll(); 
    }
    
    private function deletaRateionf($cdFilial, $nrLanctonf) {
        $sql = "DELETE RATEIONF
                WHERE CDFILIAL   = P_CDFILIAL
                AND NRLANCTONF = P_NRLANCTONF";
               
        $params = array("P_CDFILIAL"    => $cdFilial,
                        "P_NRLANCTONF"  => $nrLanctonf);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);        
    }
    
    public function getConsContaRateio($identrsaid, $cdtipooper) {
        $sql = "SELECT NVL(IDCONSCTBRAT, 'N') AS IDCONSCTBRAT
                FROM TIPOOPERAC
                WHERE IDENTRSAIDOP = :P_IDENTRSAIDOP
                AND CDTIPOOPER   = :P_CDTIPOOPER";
               
        $params = array("P_IDENTRSAIDOP"    => $identrsaid,
                        "P_CDTIPOOPER"  => $cdtipooper);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();      
    }
    
    private function getRatCt($tipoConta, $identrsaid){
        if($identrsaid == 'E'){
            $sql = "SELECT CDCENTCUST, VRPERATTPCTP, CDFILIAL
                        FROM RATTPCTP
                       WHERE CDTPCTPAGAR  = :P_TIPOCONTA";
        }else{// $identrsaid == 'S'
            $sql = "SELECT CDFILIAL, CDCENTCUST, VRPERATTPCTR
                       FROM RATTPCTR
                      WHERE CDTPCTRECEB  = :P_TIPOCONTA";
        }
               
        $params = array("P_TIPOCONTA"    => $tipoConta);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function insertRateionf($filial, $lancamento, $nrseqitprod, $filiRate, $vrrateio, $cdcentcust, $vrperattpctp) {
    	$sql = "INSERT INTO RATEIONF
                              (CDFILIAL,    NRLANCTONF, NRSEQITPROD,
                               CDFILICCUST, VRRATENF,   CDCENTCUSNF, VRPERRAT)
                           VALUES
                           (:P_FILIAL,   :P_LANCAMENTO, :P_NRSEQITPROD,
                            :P_FILIRATE, :P_VRRATEIO,   :P_CDCENTCUST, :P_VRPERATTPCTP)";
              
              
        $params = array("P_FILIAL"		=> $filial,
						"P_LANCAMENTO"		=> $lancamento, 
						"P_NRSEQITPROD"		=> $nrseqitprod,
						"P_FILIRATE"		=> $filiRate,
						"P_VRRATEIO"			=> $vrrateio,
						"P_CDCENTCUST"	=> $cdcentcust,
						"P_VRPERATTPCTP"		=> $vrperattpctp);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function InsertRatNovProd($filial, $lancamento, $nrseqitprod, $filiRate, $cdcentcust, $vrperc, $vrcontitem) {
    	$sql = "INSERT INTO RATEIONF
                  (CDFILIAL,    NRLANCTONF,  NRSEQITPROD,
                   CDFILICCUST, CDCENTCUSNF, VRRATENF, VRPERRAT)
               VALUES
               (:P_FILIAL, :P_LANCAMENTO, :P_NRSEQITPROD,
                :P_FILIRATE, :P_CDCENTCUST, ROUND(((:P_VRPERC * :P_VRCONTITEM) / 100), 2), :P_VRPERC)";
              
              
        $params = array("P_FILIAL"		    => $filial,
						"P_LANCAMENTO"		=> $lancamento, 
						"P_NRSEQITPROD"		=> $nrseqitprod,
						"P_FILIRATE"		=> $filiRate,
						"P_CDCENTCUST"	    => $cdcentcust,
						"P_VRPERC"			=> $vrperc,
						"P_VRCONTITEM"		=> $vrcontitem);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    
    public function updateRateionf($filial, $lancamento, $nrseqitprod, $filiRate, $vrrateio, $cdcentcust, $vrperattpctp) {
    	$sql = "UPDATE RATEIONF
                              SET VRRATENF = VRRATENF + :P_VRRATEIO,
                                  VRPERRAT = VRPERRAT + :P_VRPERATTPCTP
                            WHERE CDFILIAL     = :P_FILIAL
                              AND NRLANCTONF   = :P_LANCAMENTO
                              AND NRSEQITPROD  = :P_NRSEQITPROD
                              AND CDFILICCUST  = :P_FILIRATE
                              AND CDCENTCUSNF  = :P_CDCENTCUST";
              
              
        $params = array("P_FILIAL"		=> $filial,
						"P_LANCAMENTO"		=> $lancamento, 
						"P_NRSEQITPROD"		=> $nrseqitprod,
						"P_FILIRATE"		=> $filiRate,
						"P_VRRATEIO"			=> $vrrateio,
						"P_CDCENTCUST"	=> $cdcentcust,
						"P_VRPERATTPCTP"		=> $vrperattpctp);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function insertRateionfCompleto($filial, $lancamento, $cdfilirece, $idEntrSaid) {
        if($idEntrSaid == 'S'){
            $sql = "INSERT INTO RATEIONF
                     (CDFILIAL,    NRLANCTONF, NRSEQITPROD,
                      CDFILICCUST, VRRATENF,   CDCENTCUSNF, VRPERRAT)
                  (SELECT I.CDFILIAL,   I.NRLANCTONF, I.NRSEQITPROD,
                          N.CDFILIEMIT, I.VRCONTITEM,
                          NVL(NVL(NVL(PF.CDCCUSTSAIDF,P.CDCCUSTSAID),F.CDCENTCUST),E.CDCENTCUST) CDCENTCUST,
                          100 AS VRPERRAT
                   FROM NOTAFISCAL N, ITEMPRODUTO I, PRODFILI PF,
                        PRODUTO P, EMPRESA E, FILIAL F
                   WHERE N.CDFILIAL   = :P_FILIAL   AND
                         N.NRLANCTONF = :P_LANCAMENTO AND
                         I.CDFILIAL   = :P_FILIAL   AND
                         I.NRLANCTONF = :P_LANCAMENTO AND
                         I.CDPRODUTO  = PF.CDPRODUTO  AND
                         PF.CDFILIAL  = N.CDFILIEMIT  AND
                         I.CDPRODUTO  = P.CDPRODUTO   AND
                         N.CDEMPRESA  = E.CDEMPRESA   AND
                         N.CDFILIEMIT = F.CDFILIAL    AND
                         NVL(NVL(NVL(PF.CDCCUSTSAIDF,P.CDCCUSTSAID),F.CDCENTCUST),E.CDCENTCUST) IS NOT NULL)";
                         
            $params = array("P_FILIAL"		    => $filial,
					    	"P_LANCAMENTO"		=> $lancamento);
        }
        else{
        	$sql = "INSERT INTO RATEIONF
                      (CDFILIAL,    NRLANCTONF, NRSEQITPROD,
                       CDFILICCUST, VRRATENF,   CDCENTCUSNF, VRPERRAT)
                   (SELECT I.CDFILIAL,   I.NRLANCTONF, I.NRSEQITPROD,
                           N.CDFILIRECE, I.VRCONTITEM,
                           NVL(NVL(NVL(PF.CDCCUSTENTRF,P.CDCCUSTENTR),F.CDCENTCUST),E.CDCENTCUST) CDCENTCUST,
                           100 AS VRPERRAT
                    FROM NOTAFISCAL N, ITEMPRODUTO I, PRODFILI PF,
                         PRODUTO P, EMPRESA E, FILIAL F
                    WHERE N.CDFILIAL   = :P_FILIAL   AND
                          N.NRLANCTONF = :P_LANCAMENTO AND
                          I.CDFILIAL   = :P_FILIAL   AND
                          I.NRLANCTONF = :P_LANCAMENTO AND
                          I.CDPRODUTO  = PF.CDPRODUTO(+)  AND
                          PF.CDFILIAL(+)  = NVL(I.CDFILIMOVI,:P_CDFILIRECE)  AND
                          I.CDPRODUTO  = P.CDPRODUTO   AND
                          N.CDEMPRESA  = E.CDEMPRESA   AND
                          N.CDFILIRECE = F.CDFILIAL  AND
                          NVL(NVL(NVL(PF.CDCCUSTENTRF,P.CDCCUSTENTR), F.CDCENTCUST),E.CDCENTCUST) IS NOT NULL)";
            
            $params = array("P_FILIAL"		    => $filial,
					    	"P_LANCAMENTO"		=> $lancamento, 
					    	"P_CDFILIRECE"		=> $cdfilirece);
        }
              
        
        
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getTipoContNF($filial, $lancamento, $idEntrSaid){
        if($idEntrSaid == 'S'){
            $sql = "SELECT TR.CDTPCTRECEB, TR.CDCLASSFINA
                   FROM (SELECT MAX(CDARVPROD) AS CDARVPROD
                         FROM ITEMPRODUTO I, PRODUTO P
                         WHERE CDFILIAL    = :P_FILIAL AND
                               NRLANCTONF  = :P_LANCAMENTO AND
                               I.CDPRODUTO = P.CDPRODUTO
                         GROUP BY VRTOTAITEM
                         HAVING VRTOTAITEM = (SELECT MAX(VRTOTAITEM)
                                              FROM ITEMPRODUTO
                                              WHERE CDFILIAL    = :P_FILIAL AND
                                                    NRLANCTONF  = :P_LANCAMENTO)) IT,
                         PRODUTO PR , TIPOCTRECEBER TR
                   WHERE TR.CDTPCTRECEB = PR.CDTPCTRECEB AND
                         PR.CDARVPROD = (SELECT MAX(P.CDARVPROD)
                                         FROM TIPOCTRECEPROD C, PRODUTO P
                                         WHERE C.CDPRODUTO = P.CDPRODUTO AND
                                               (INSTR(IT.CDARVPROD, RTRIM(P.CDARVPROD)) = 1))";
        }else{// $idEntrSaid == 'E'
          $sql = "SELECT TR.CDTPCTPAGAR
                    FROM (SELECT MAX(CDARVPROD) AS CDARVPROD
                            FROM ITEMPRODUTO I, PRODUTO P
                           WHERE CDFILIAL    = :P_FILIAL AND
                                 NRLANCTONF  = :P_LANCAMENTO AND
                                 I.CDPRODUTO = P.CDPRODUTO
                           GROUP BY VRTOTAITEM
                           HAVING VRTOTAITEM = (SELECT MAX(VRTOTAITEM)
                                                  FROM ITEMPRODUTO
                                                 WHERE CDFILIAL    = :P_FILIAL AND
                                                       NRLANCTONF  = :P_LANCAMENTO)) IT,
                         PRODUTO PR , TIPOCTPAGAR TR
                   WHERE TR.CDTPCTPAGAR = PR.CDTPCTPAGAR AND
                         PR.CDARVPROD = (SELECT MAX(P.CDARVPROD)
                                           FROM TIPOCTRECEPROD C, PRODUTO P
                                          WHERE C.CDPRODUTO = P.CDPRODUTO AND
                                                (INSTR(IT.CDARVPROD, RTRIM(P.CDARVPROD)) = 1))";
        }
               
        $params = array("P_FILIAL"	        => $filial,
						"P_LANCAMENTO"		=> $lancamento);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function getTotalCont($lancamento, $filial){
        $sql = "SELECT SUM(ROUND(VRCONTITEM,2)) AS VRCONTITEM, NRSEQITPROD
                FROM ITEMPRODUTO
               WHERE NRLANCTONF  = :P_LANCAMENTO
                 AND CDFILIAL    = :P_FILIAL
               GROUP BY NRSEQITPROD";
               
        $params = array("P_LANCAMENTO"		=> $lancamento,
                        "P_FILIAL"		    => $filial);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function getSumRateio($lancamento, $filial, $nrseqitem){
        $sql = "SELECT SUM(VRRATENF) AS VRRATEIONF
                   FROM RATEIONF
                  WHERE NRLANCTONF  = :P_LANCAMENTO
                    AND CdFILIAL    = :P_FILIAL
                    AND NRSEQITPROD = :P_NRSEQITPROD";
               
        $params = array("P_LANCAMENTO"		=> $lancamento,
                        "P_FILIAL"		    => $filial,
						"P_NRSEQITPROD"		=> $nrseqitem);
               
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function updateRateionfRound($difCont, $filial, $lancamento, $nrseqitprod) {
    	$sql = "UPDATE RATEIONF
                  SET VRRATENF    = VRRATENF + ROUND(:P_DIFCONT, 2)
                WHERE CDFILIAL    = :P_CDFILIAL
                  AND NRLANCTONF  = :P_LANCAMENTO
                  AND NRSEQITPROD = :P_NRSEQITPROD
                  AND CDFILICCUST||CDCENTCUSNF = (SELECT MAX(CDFILICCUST||CDCENTCUSNF)
                                                    FROM RATEIONF
                                                   WHERE CDFILIAL    = P_CDFILIAL
                                                     AND NRLANCTONF  = P_LANCAMENTO
                                                     AND NRSEQITPROD = P_NRSEQITPROD
                                                     AND VRRATENF    = (SELECT MAX(VRRATENF)
                                                                          FROM RATEIONF
                                                                         WHERE CDFILIAL    = P_CDFILIAL
                                                                           AND NRLANCTONF  = P_LANCAMENTO
                                                                           AND NRSEQITPROD = P_NRSEQITPROD))";
              
              
        $params = array("P_DIFCONT"		    => $difCont,
                        "P_CDFILIAL"        => $filial,
						"P_LANCAMENTO"		=> $lancamento, 
						"P_NRSEQITPROD"		=> $nrseqitprod);
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function buscaNotaOl($filial, $lancamento) {
    	$sql = "SELECT IT.CDFILIALIT,
                  IT.NRLANCTONFIT,
                  IP.CDPRODUTO
                FROM ITEMNOTAFISC IT,
                  ITEMPRODUTO IP
                WHERE IT.CDFILIAL = :P_CDFILIAL
                AND IT.NRLANCTONF = :P_NRLANCTONF
                AND IP.CDFILIAL   = IT.CDFILIAL
                AND IP.NRLANCTONF = IT.NRLANCTONF";
              
		$params = array("P_CDFILIAL"          => $filial,
						"P_NRLANCTONF"		    => $lancamento);	
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll();  
    }
    
    public function updateNotaOl($filial, $lancamento, $cdProduto) {
    	$sql = "UPDATE ITEMPRODUTO
                SET QTDEVOLUCAOOL = 0,
                  IDDEVOLUCAOOL   = 'N',
                  CDMOTIVODEV     = ''
                WHERE CDFILIAL    = :P_CDFILIAL
                AND NRLANCTONF    = :P_NRLANCTONF
                AND CDPRODUTO     = :P_CDPRODUTO";
              
		$params = array("P_CDFILIAL"            => $filial,
						"P_NRLANCTONF"		    => $lancamento,
						"P_CDPRODUTO"           => $cdProduto);	
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
		
    }
    
     public function getParamManifesta() {
    	$sql = "SELECT IDCIENCIAEMISS, IDCONFIRMAEMISS
    			FROM PARAMMANIFESTA
    			WHERE  ((IDCIENCIAEMISS = '2') OR  (IDCONFIRMAEMISS = '2'))";
       
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
		return $std->fetchAll(); 
    }
    
    public function getNotaDestinada($nrAcessoNfe) {
		$sql = "SELECT NVL(IDSTATUSMANIF, '0') IDSTATUSMANIF
		        FROM NOTADESTINADA
		       WHERE NRACESSONFE = :P_NRACESSONFE";
		
		$params = array("P_NRACESSONFE"		=> $nrAcessoNfe);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
	}
	
	public function buscaServico($cdFilial, $cdServFili) {
	    $sql = "SELECT NMSERVICO
                FROM SERVICO
                WHERE CDFILIAL = :P_CDFILIAL
                AND CDSERVFILI = :P_CDSERVFILI";
        
        $params = array("P_CDFILIAL"		=> $cdFilial,
                        "P_CDSERVFILI"      => $cdServFili);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
	}
	
	public function aprovaNfXmlAutomatica() {
        $sql = "SELECT IDPERAPROVNFXML FROM PARAMGERAL";
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute();
		return $std->fetch()["IDPERAPROVNFXML"] == 'S' ? true : false; 
    }
	
    public function getAprovAutops($cdtipooper) {
        $sql = "SELECT IDAPROVAUTOPS 
        FROM TIPOOPERAC 
        WHERE IDENTRSAIDOP = 'E' 
        AND CDTIPOOPER = :P_CDTIPOOPER";
			
		$params = array( "P_CDTIPOOPER" => $cdtipooper );
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		return $std->fetch()["IDAPROVAUTOPS"] == 'S' ? true : false; 
    }
    
    public function getIdrearatmannf($cdtipooper) {
        $sql = "SELECT IDREARATMANNF 
        FROM TIPOOPERAC 
        WHERE IDENTRSAIDOP = 'E' 
        AND CDTIPOOPER = :P_CDTIPOOPER";
			
		$params = array( "P_CDTIPOOPER" => $cdtipooper );
					
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        
		return $std->fetch()["IDREARATMANNF"]; 
    }
    
    public function getTotalImpostosItens($filial, $lancamento) {
        $sql = "SELECT SUM(T.VRIMPOSIT) VRIMPOSTOT,
                  T.CDIMPOSTOIT AS CDIMPOSTOT,
                  T.VRALIQIMPOIT,
                  CDFILIMPOIT  AS CDFILIMPTOT,
                  NRLANCIMPOIT AS NRLANCIMPTOT
                FROM TRABITPROIMPOS T,
                  IMPOSTO I
                WHERE CDFILIMPOIT = :P_CDFILIAL
                AND NRLANCIMPOIT  = :P_NRLANCTONF
                AND IDPRODNOTA   <> 'C'
                AND T.CDIMPOSTOIT = I.CDIMPOSTO
                GROUP BY CDIMPOSTOIT,
                  CDFILIMPOIT,
                  NRLANCIMPOIT,
                  T.VRALIQIMPOIT
                ORDER BY CDIMPOSTOIT";
                
        $params = array("P_CDFILIAL"		=> $filial,
                        "P_NRLANCTONF"      => $lancamento);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
    }
    
        
    public function getTotalImpostosItensSemAliquota($filial, $lancamento) {
        $sql = "SELECT SUM(T.VRIMPOSIT) VRIMPOSTOT,
                  T.CDIMPOSTOIT AS CDIMPOSTOT,
                  CDFILIMPOIT  AS CDFILIMPTOT,
                  NRLANCIMPOIT AS NRLANCIMPTOT
                FROM TRABITPROIMPOS T,
                  IMPOSTO I
                WHERE CDFILIMPOIT = :P_CDFILIAL
                AND NRLANCIMPOIT  = :P_NRLANCTONF
                AND IDPRODNOTA   <> 'C'
                AND T.CDIMPOSTOIT = I.CDIMPOSTO
                GROUP BY CDIMPOSTOIT,
                  CDFILIMPOIT,
                  NRLANCIMPOIT
                ORDER BY CDIMPOSTOIT";
                
        $params = array("P_CDFILIAL"		=> $filial,
                        "P_NRLANCTONF"      => $lancamento);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
    }
    
    public function validaExisteItem($filial, $lancamento) {
        $sql = "SELECT COUNT(CDPRODUTO) PRODUTOS
                FROM ITEMPRODUTO
                WHERE CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF";
                
        $params = array("P_CDFILIAL"		=> $filial,
                        "P_NRLANCTONF"      => $lancamento);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetch();
		return ($result["PRODUTOS"] > 0) ? true : false ; 
    }
    
    public function validaLocalProduto($filial, $lancamento) {
        $sql = "SELECT
                    PR.CDARVPROD,
                    PR.NMPRODUTO,
                    P.IDUTILALMOX,
                    IP.CDALMOXARIFE,
                    P.IDUTILLCESTQ,
                    IP.CDLOCALESTOQ,
                    P.IDUTILLOTE,
                    IP.NRLOTEESTQ,
                    PR.CDPRODESTO,
                    IP.NRSEQITPROD
                FROM
                    PARAMFILIAL   P,
                    NOTAFISCAL    N,
                    PRODUTO       PR,
                    ITEMPRODUTO   IP,
                    PRODFILI      PF
                WHERE
                    P.IDCTRLESTQ = 'S'
                    AND PF.IDCNTRESTOQ = 'S'
                    AND PF.CDPRODUTO = PR.CDPRODUTO
                    AND ( ( ( ( IP.CDLOCALESTOQ IS NULL )
                              OR ( IP.CDLOCALESTOQ = ' ' ) )
                            AND ( P.IDUTILLCESTQ = 'S' ) )
                          OR ( ( ( IP.CDALMOXARIFE IS NULL )
                              OR (IP.CDALMOXARIFE = ' ') )
                               AND ( P.IDUTILALMOX = 'S' ) )
                          OR ( ( ( IP.NRLOTEESTQ IS NULL )
                               AND ( P.IDUTILLOTE = 'S' )
                              OR (IP.NRLOTEESTQ = ' ') )
                               AND ( PF.IDUTCTRLLOTE = 'S' ) ) )
                    AND IP.CDPRODUTO = PR.CDPRODUTO
                    AND P.CDFILIAL = DECODE(N.IDENTRSAIDOP, 'S', CDFILIEMIT, CDFILIRECE)
                    AND IP.CDFILIAL = N.CDFILIAL
                    AND IP.NRLANCTONF = N.NRLANCTONF
                    AND PF.CDFILIAL = DECODE(N.IDENTRSAIDOP, 'S', CDFILIEMIT, CDFILIRECE)
                    AND N.CDFILIAL = :P_CDFILIAL
                    AND N.NRLANCTONF = :P_NRLANCTONF";
                
        
        $params = array("P_CDFILIAL"		=> $filial,
                        "P_NRLANCTONF"      => $lancamento);
                
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
    }
    
    public function tipoOperacAtuEstoque($cdtipooper, $identrsaid){
      $sql = "SELECT TP.IDATUESTTPOP 
        FROM TIPOOPERAC TP 
        WHERE TP.CDTIPOOPER = :P_CDTIPOOPER
        AND TP.IDENTRSAIDOP = :P_IDENTRSAIDOP";
                
      $params = array(
        "P_CDTIPOOPER" => $cdtipooper,
        "P_IDENTRSAIDOP" => $identrsaid
      );
              
	    $std = $this->entityManager->getConnection()->prepare($sql);
      $std->execute($params);
      $result = $std->fetchAll();
      
	    return $result[0]["IDATUESTTPOP"] == 'S' ? true : false; 
    }
    
    public function getProdutosInativos($filial, $lancamento) {
        $sql = "SELECT P.CDARVPROD, P.CDPRODESTO, P.NMPRODUTO, PF.IDPRODATIV, PP.IDPRODATIVPR AS PRODESTOQATIVO, P.IDPRODATIVPR, IT.NRSEQITPROD FROM NOTAFISCAL N, ITEMPRODUTO IT, PRODUTO P, PRODUTO PP, PRODFILI PF
                WHERE (P.IDPRODATIVPR = 'N'
                OR PP.IDPRODATIVPR = 'N'
                OR PF.IDPRODATIV  = 'I' )
                AND PF.CDFILIAL   = IT.CDFILIAL
                AND PF.CDPRODUTO  = P.CDPRODUTO
                AND P.CDPRODESTO  = PP.CDPRODUTO
                AND P.CDPRODUTO   = IT.CDPRODUTO
                AND N.CDFILIAL    = IT.CDFILIAL
                AND N.NRLANCTONF  = IT.NRLANCTONF
                AND IT.CDFILIAL   = :P_CDFILIAL
                AND IT.NRLANCTONF = :P_NRLANCTONF";
                
        $params = array("P_CDFILIAL"		=> $filial,
                        "P_NRLANCTONF"      => $lancamento);
                
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetchAll(); 
    }
    
    public function verificaNotaSendoUtilizada($filial, $lancamento) {
        $sql = "SELECT CDFILIAL
                FROM NOTAFISCAL
                WHERE CDFILIAL    = :P_CDFILIAL
                AND NRLANCTONF    = :P_NRLANCTONF  FOR UPDATE NOWAIT";
                
        $params = array(
            "P_NRLANCTONF"        => $lancamento,
            "P_CDFILIAL"          => $filial
        );
        
        try {
            $std = $this->entityManager->getConnection()->prepare($sql);
            $std->execute($params);
            $result = $std->fetch();
            return false;
        }
        catch(\Exception $e) {
            return true;
        }
    }
    
    public function getItensNota($filial, $lancamento) {
        $sql = "SELECT
                    CDARVPROD,
                    NRSEQITPROD,
                    VRTOTAITEM
                FROM
                    ITEMPRODUTO   IP,
                    PRODUTO       P
                WHERE
                    IP.CDPRODUTO = P.CDPRODUTO
                    AND CDFILIAL = :P_CDFILIAL
                    AND NRLANCTONF = :P_NRLANCTONF";
                
        $params = array(
            "P_NRLANCTONF"        => $lancamento,
            "P_CDFILIAL"          => $filial
        );
        
       $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function getNrLancestq($cdfilial, $nrlanctonf){
        $params = array(
            "P_CDFILIAL"   => $cdfilial,
            "P_NRLANCTONF" => $nrlanctonf
        );
        
        $sql = "SELECT NRLANCESTQ
                 FROM LANCTOESTOQ
                 WHERE CDFILIAL = :P_CDFILIAL
                 AND NRLANCTONF = :P_NRLANCTONF";
                 
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetch();
        if($result){
            return $result["NRLANCESTQ"];
        } else{
            return null;  
        }
    }
    
    public function getItensLancEstq($cdfilial, $nrlancestq){
        $params = array(
            "P_CDFILIAL"   => $cdfilial,
            "P_NRLANCTOEST" => $nrlancestq   
        );
        
        $sql = "SELECT CDALMOXARIFE, CDLOCALESTOQ, NRLOTEESTQ,
                       NRSUBLOTE, CDPRODMOVI, DTLANCMOVI
                FROM ITLANCTOEST
                WHERE   CDFILIAL   = :P_CDFILIAL
                    AND NRLANCESTQ = :P_NRLANCTOEST";

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    
    public function deleteCustoCmProd($itens, $cdfilial){
        $connection = $this->entityManager->getConnection();
        $sql = "DELETE CUSTOCMPPROD
                WHERE CDFILLANCEST = :P_CDFILLANCEST
                 AND CDALMOXARIFE = :P_CDALMOXARIFE
                 AND CDLOCALESTOQ = :P_CDLOCALESTOQ
                 AND NRLOTEESTQ = :P_NRLOTEESTQ
                 AND NRSUBLOTE = :P_NRSUBLOTE
                 AND CDPRODUTO = :P_CDPRODUTO
                 AND DTCUSTCMPPRO = :P_DTCUSTCMPPRO";
        
        foreach($itens as $item){
            $params = array(
                "P_CDFILLANCEST" => $cdfilial,
                "P_CDALMOXARIFE" => $item["CDALMOXARIFE"],
                "P_CDLOCALESTOQ" => $item["CDLOCALESTOQ"],
                "P_NRLOTEESTQ" => $item["NRLOTEESTQ"],
                "P_NRSUBLOTE" => $item["NRSUBLOTE"],
                "P_CDPRODUTO" => $item["CDPRODMOVI"],
                "P_DTCUSTCMPPRO" => $item["DTLANCMOVI"]
            );
            
            $std = $connection->prepare($sql);
            $std->execute($params);
        }
        
    }
    
    public function alteraDtLancto($dtentrsaid, $cdfilial, $nrlancestq){
        $connection = $this->entityManager->getConnection();
        $params = array(
            "P_DTENTRSAID" => $dtentrsaid,
            "P_CDFILIAL" => $cdfilial,
            "P_NRLANCTOEST" => $nrlancestq
        );
        $sql = "UPDATE LANCTOESTOQ
                SET DTLANCESTQ   = :P_DTENTRSAID
                WHERE CDFILIAL   = :P_CDFILIAL
                  AND NRLANCESTQ = :P_NRLANCTOEST";
                  
        $std = $connection->prepare($sql);
        $std->execute($params);
        
        $sql = "UPDATE ITLANCTOEST
                SET IDEVOLESTOQ = 'S', DTLANCMOVI = :P_DTENTRSAID
                WHERE CDFILIAL   = :P_CDFILIAL
                  AND NRLANCESTQ = :P_NRLANCTOEST";
                  
        $std = $connection->prepare($sql);
        $std->execute($params);
        
    }
    
    public function getDefaultNovaNota($cdFilial, $cdOperador, $idEntrSaid) {
        $sql = "SELECT FO.CDOPERADOR,
                  F.NMFILIAL,
                  F.CDEMPRESA,
                  E.NMEMPRESA,
                  F.CDINSCESTA,
                  F.CDINSCESTA AS IEDEST,
                  F.CDINSCESTA AS IEREM,
                  F.CDINSCMUNI,
                  F.NRORG,
                  F.IDTPIJURFILI,
                  F.NRINSJURFILI,
                  PF.IDUTBLOQOPAFPF,
                  PF.IDBLOQCMPSAFFIL,
                  NVL(PF.NRTAMNUMNFUTDFE, 9) AS NRTAMNUMNFUTDFE,
                  PF.IDCANCEXTMPNFE,
                  PF.DSPARFILFIS,
                  DECODE(PF.NRMAXDIAANTDTAF, NULL, 'T', 'N')                                                                                        AS DTMINVAZIA,
                  DECODE(PF.NRMAXDIAANTDTAF, NULL, NVL(PF.DTPROCESSA, TO_DATE(SYSDATE)), NVL(PF.DTPROCESSA, TO_DATE(SYSDATE)) - PF.NRMAXDIAANTDTAF) AS DTPROCINI,
                  DECODE(PF.NRMAXDIAPOSDTAF, NULL, 'T', 'N')                                                                                        AS DTMAXVAZIA,
                  DECODE(PF.NRMAXDIAPOSDTAF, NULL, NVL(PF.DTPROCESSA, TO_DATE(SYSDATE)), NVL(PF.DTPROCESSA, TO_DATE(SYSDATE)) + PF.NRMAXDIAPOSDTAF) AS DTPROCFIN,
                  PG.IDBLOQITNFAPUR,
                  PG.IDOBRCHAVACES,
                  PG.IDPELANIMPXML,
                  DECODE(SUBSTR(PF.DSPARFILFIS, 2, 1), 'S', 'S', PG.IDBLOQCMPSAF) AS IDBLOQAF,
                  DECODE(PF.IDUTBLOQOPAFPF, 'N', PG.IDUTBLOQOPAFPG, 'S')          AS IDUTBLOQOPAF,
                  PG.IDUTBLOQOPAFPG,
                  PG.IDBLOQCMPSAF,
                  NVL(PG.IDGERNFIGUALXML, 'N') AS IDGERNFIGUALXML,
                  EN.CDPAIS,
                  EN.SGESTADO,
                  M.NMMUNICIPIO,
                  EN.DSENDEFILI,
                  EN.NMBAIRFILI,
                  EN.NRCEPFILI,
                  EN.NRTELEFILI,
                  NVL(PI.IDPERIMPSPRAF, 'N') AS IDPERIMPSPRAF,
                  F.IDFUNCAOFILI,
                  PG.IDBLOQALTPROXML,
                  PF.DTPROCESSA,
                  DECODE(:P_IDENTRSAIDOP, 'E', PF.CDFILIAL, '') AS CDFILIRECE,
                  DECODE(:P_IDENTRSAIDOP, 'S', PF.CDFILIAL, '') AS CDFILIEMIT,
                  :P_IDENTRSAIDOP AS IDENTRSAIDOP
                FROM FILIOPER FO
                JOIN FILIAL F
                ON FO.CDFILIAL    = F.CDFILIAL
                AND FO.CDOPERADOR = :P_CDOPERADOR
                LEFT JOIN EMPRESA E
                ON E.CDEMPRESA = F.CDEMPRESA
                AND E.NRORG    = F.NRORG
                INNER JOIN PARAMFILIAL PF
                ON PF.CDFILIAL = F.CDFILIAL
                INNER JOIN PARAMGERAL PG
                ON PG.NRORG = F.NRORG
                JOIN ENDEFILI EN
                ON F.CDFILIAL       = EN.CDFILIAL
                AND F.NRORG         = EN.NRORG
                AND EN.IDTPENDEFILI = 'P'
                JOIN MUNICIPIO M
                ON E.CDPAIS       = M.CDPAIS
                AND E.SGESTADO    = M.SGESTADO
                AND E.CDMUNICIPIO = M.CDMUNICIPIO
                LEFT JOIN PARAMINTEGRA PI
                ON CDPARAMINT = '1'
                WHERE F.CDFILIAL = :P_CDFILIAL
                ORDER BY F.CDFILIAL";
                
        $params = array(
            "P_CDFILIAL"        => $cdFilial,
            "P_CDOPERADOR"      => $cdOperador,
            "P_IDENTRSAIDOP"    => $idEntrSaid
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();
    }
    
    public function getItemNotaFisc($cdfilial, $nrlanctonf) {
        $sql = "SELECT CDFILIALIT, NRLANCTONFIT FROM ITEMNOTAFISC WHERE CDFILIAL = :P_CDFILIAL AND NRLANCTONF = :P_NRLANCTONF";
        
        $params = array(
            "P_CDFILIAL"        => $cdfilial,
            "P_NRLANCTONF"      => $nrlanctonf
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();
    }
    
    public function getErrosAvisosItem($item, $nrorg) {
        $sql = "SELECT CDPRODUTO, IDERROSITEM, IDAVISOSITEM FROM ITEMPRODUTO WHERE CDFILIAL = :P_CDFILIAL AND NRLANCTONF = :P_NRLANCTONF AND CDPRODUTO = :P_CDPRODUTO AND NRSEQITPROD = :P_NRSEQITPROD AND NRORG = :P_NRORG";
        
        $params = array (
            "P_CDFILIAL"   => $item->getCdfilial(),
            "P_NRLANCTONF" => $item->getNrlanctonf(),
            "P_CDPRODUTO"  => $item->getCdproduto(),
            "P_NRSEQITPROD"=> $item->getNrseqitprod(),
            "P_NRORG"      => $nrorg
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
		return $std->fetch();
    }
    
    public function valorTotalDeItens($filial, $lancamento) {
        $sql = "SELECT
            SUM(VRTOTAITEM) VRTOTAITENS
        FROM
            ITEMPRODUTO
        WHERE
            CDFILIAL = :P_CDFILIAL
            AND NRLANCTONF = :P_NRLANCTONF";
                
        $params = array("P_CDFILIAL"		=> $filial,
                        "P_NRLANCTONF"      => $lancamento);
		       
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetch();
		return $result["VRTOTAITENS"];
    }
    
    public function getProdutoAfsImportadas($cdfilial ,$nrlanctonf , $nrorg) {
        $sql = 'SELECT 
                    A.CDFILLANC,
                    N.NRLANCTONF,
                    N.CDFILIAL,
                    I.CDPRODUTO,
                    IA.VRINDICEITMAF AS VRPREUNIAF,
                    IA.VRDESCITMAF,
                    A.CDINDICEAF AS CDINDICE,
                    A.NRAUTOFORN,
                    IA.VRPERDESAF,
                    IA.QTEFETENTRAF,
                    IA.QTPROGENTAF,
                    IA.IDBAIXITMAF
                FROM
                    ITEMPRODUTO I,
                    ITEMAUTF  IA,
                    AUTOFORNE  A,
                    NOTAFISCAL N
                WHERE
                    IA.NRAUTOFORN = A.NRAUTOFORN
                    AND IA.CDPRODUTO = I.CDPRODUTO
                    AND I.NRLANCTONF = N.NRLANCTONF 
                    AND A.NRAUTOFORN = I.NRAUTOFORN
                    AND A.NRORG = :NRORG
                    AND A.CDFILLANC = :P_CDFILIAL
                    AND N.CDFILIAL = :P_CDFILIAL
                    AND N.NRLANCTONF = :P_NRLANCTONF';
                    
        $params = array(
                    "P_CDFILIAL" => $cdfilial,
                    "P_NRLANCTONF" => $nrlanctonf,
                    "NRORG"    => $nrorg
                );
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
        
    }
    
     public function updateValoresProdutosNF ($cdproduto, $nmfilial, $nrlanctonf ,$qtitemprod, $vrunititem, $vrdescitem, $vrtotaitem , $nrorg){
        $sql = "UPDATE ITEMPRODUTO
                SET
                    VRTOTAITEM = :P_VRTOTAITEM,
                    VRDESCITEM = :P_VRDESCITEM,
                    VRUNITITEM = :P_VRUNITITEM,
                    QTITEMPROD = :P_QTITEMPROD
                WHERE
                    CDFILIAL = :P_CDFILIAL
                    AND NRLANCTONF = :P_NRLANCTONF
                    AND CDPRODUTO = :P_CDPRODUTO
                    AND NRORG = :P_NRORG";
        
    $params = array("P_VRTOTAITEM"      => $vrtotaitem,
                        "P_VRDESCITEM"  => $vrdescitem,
                        "P_VRUNITITEM"  => $vrunititem,
                        "P_QTITEMPROD"  => $qtitemprod,
                        "P_CDFILIAL"    =>$nmfilial,
                        "P_NRLANCTONF"  =>$nrlanctonf,
                        "P_CDPRODUTO"   => $cdproduto,
                        "P_NRORG"       =>$nrorg
                        );
						
		$std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function getQuantidadeProduto ($cdfilial, $nrlanctonf, $cdproduto, $nrorg, $nrautoforn) {
        $sql ='SELECT
                    QTNFEFETAF
                FROM
                    NFRELACAF
                WHERE
                    CDFILIAL = :P_CDFILIAL
                    AND NRLANCTONF = :P_NRLANCTONF
                    AND NRAUTOFORN = :P_NRAUTOFORN
                    AND CDPRODUTO = :P_CDPRODUTO
                    AND NRORG = :NRORG';
                
     $params = array(
         
        "P_CDFILIAL"   => $cdfilial, 
        "P_NRAUTOFORN" => $nrautoforn,
        "P_NRLANCTONF" => $nrlanctonf,
        "P_CDPRODUTO" => $cdproduto,
        "NRORG"    => $nrorg

        );
                
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetch();
		return $result["QTNFEFETAF"];
                
    }

    public function updateDtVencimento($cdfilial, $nrlanctonf, $nrseqvenrec, $dtvenrec) {
        $sql = "UPDATE CONTARECEBER
                SET
                    DTATUAVENREC = :P_DTATUAVENREC,
                    DTORIGVENREC = :P_DTORIGVENREC
                WHERE
                    CDFILIAL = :P_CDFILIAL
                    AND NRLANCTONF = :P_NRLANCTONF
                    AND NRSEQVENREC = :P_NRSEQVENREC";
                    
        $params = array(
            "P_CDFILIAL"        => $cdfilial, 
            "P_NRLANCTONF"      => $nrlanctonf,
            "P_NRSEQVENREC"     => $nrseqvenrec,
            "P_DTATUAVENREC"    => $dtvenrec,
            "P_DTORIGVENREC"    => $dtvenrec
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    /*********************** Apaga relacionamento nota de origem ******************************/

    public function deletaRelacNotaOrigem($cdfilial, $nrLanctoNF){
        $sql =
            "DELETE FROM 
                ITEMNOTAFISC
            WHERE
                NRLANCTONF = :P_NRLANCTONF
                AND CDFILIAL = :P_CDFILIAL";

        $params = array(
            'P_NRLANCTONF' => $nrLanctoNF,
            'P_CDFILIAL' => $cdfilial
        );
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function getAliqtempprod($nrLanctoNF, $organizacao){
        $sql =
            "SELECT
                CDIMPOTEMP,
                VRALIQUTMP,
                VRADICALIQFECP
            FROM
                ALIQTEMPPROD
            WHERE
                NRALIQTEMP = :P_NRALIQTEMP
                AND NRORG = :P_NRORG";

        $params = array(
            ':P_NRALIQTEMP' => $nrLanctoNF,
            ':P_NRORG' => $organizacao
        );
        
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function deleteAliqtempprod($nrLanctoNF, $cdimpotemp ,$organizacao){
        $sql =
            "DELETE FROM 
                ALIQTEMPPROD
            WHERE
                CDIMPOTEMP = :P_CDIMPOTEMP
                AND NRALIQTEMP = :P_NRALIQTEMP
                AND NRORG = :P_NRORG";

       $params = array(
            ':P_NRALIQTEMP' => $nrLanctoNF,
            ':P_NRORG' => $organizacao,
            'P_CDIMPOTEMP' => $cdimpotemp
        );
        
        $query = $this->entityManager->getConnection()->prepare($sql);
        $query->execute($params);
    }
    
    public function updateItemImposTemp($params) {
        $sql = "UPDATE TRABITPROIMPOS
                SET
                    VRADICALIQFECP = :P_VRADICALIQFECP
                WHERE
                    CDFILIMPOIT = :P_CDFILIMPOIT 
                    AND NRLANCIMPOIT = :P_NRLANCIMPOIT 
                    AND NRSEQIMPOIT = :P_NRSEQIMPOIT 
                    AND CDIMPOSTOIT = :P_CDIMPOSTOIT 
                    AND NRSEQITPRO = :P_NRSEQITPRO
                    AND NRORG = :P_NRORG
                    AND CDOPERINCLUSAO = :P_CDOPERINCLUSAO";
    
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
    }
    
    public function getAllItemproimpos($params){
        $sql =
            "SELECT
                *
            FROM
                ITEMPROIMPOS
            WHERE
                CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF
                AND NRSEQITPROD = :P_NRSEQITPROD
                AND CDIMPOSTO = :P_CDIMPOSTO
                AND NRITPROIMPOS = :P_NRITPROIMPOS
                AND NRORG = :P_NRORG";

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }
    
    public function getdsInfadicprod($params) {
        $sql =
            "SELECT DSINFADICPROD
               FROM ITEMPRODUTO
              WHERE CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF
                AND NRSEQITPROD = :P_NRSEQITPROD
                AND NRORG = :P_NRORG";

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        $result = $std->fetch();
		return $result["DSINFADICPROD"];
    }
    
    public function insereImpostoItem ($row) {
         $sql =
            "INSERT 
            INTO
                ITEMPROIMPOS
                (   
                    CDBENEFFISCAL,
                    CDCEST,
                    CDENQLEGALIPI,
                    CDFILIAL,
                    CDFISCALIT,
                    CDFISCOPERIT,
                    CDIMPOSTO,
                    CDREGRAIT,
                    CDSITUTRIBIT,
                    CDSITUTRIBITNFE,
                    IDCONSDESONVRNF,
                    IDMODBCITNFE,
                    IDMODBCSTITNFE,
                    IDMOTDESONERAIT,
                    NRITPROIMPOS,
                    NRLANCTONF,
                    NRSEQITPROD,
                    VRADICALIQFECP,
                    VRADICFECP,
                    VRALIQITEM,
                    VRBASECALCIT,
                    VRCONTITIMP,
                    VRIMPOSTODESON,
                    VRIMPOSTOIT,
                    VRPARDEDIT,
                    VRPERMAJBCIT,
                    VRPERREDBCIT
                )
            VALUES
                (
                    :P_CDBENEFFISCAL,
                    :P_CDCEST,
                    :P_CDENQLEGALIPI,
                    :P_CDFILIAL,
                    :P_CDFISCALIT,
                    :P_CDFISCOPERIT,
                    :P_CDIMPOSTO,
                    :P_CDREGRAIT,
                    :P_CDSITUTRIBIT,
                    :P_CDSITUTRIBITNFE,
                    :P_IDCONSDESONVRNF,
                    :P_IDMODBCITNFE,
                    :P_IDMODBCSTITNFE,
                    :P_IDMOTDESONERAIT,
                    :P_NRITPROIMPOS,
                    :P_NRLANCTONF,
                    :P_NRSEQITPROD,
                    :P_VRADICALIQFECP,
                    :P_VRADICFECP,
                    :P_VRALIQITEM,
                    :P_VRBASECALCIT,
                    :P_VRCONTITIMP,
                    :P_VRIMPOSTODESON,
                    :P_VRIMPOSTOIT,
                    :P_VRPARDEDIT,
                    :P_VRPERMAJBCIT,
                    :P_VRPERREDBCIT)";
         
        $params = 
            array("P_CDBENEFFISCAL" => $row["CDBENEFFISCAL"],
                "P_CDCEST" => $row["CDCEST"],
                "P_CDENQLEGALIPI" => $row["CDENQLEGALIPI"],
                "P_CDFILIAL" => $row["CDFILIAL"],
                "P_CDFISCALIT" => $row["CDFISCALIT"],
                "P_CDFISCOPERIT" => $row["CDFISCOPERIT"],
                "P_CDIMPOSTO" => $row["CDIMPOSTO"],
                "P_CDREGRAIT" => $row["CDREGRAIT"],
                "P_CDSITUTRIBIT" => $row["CDSITUTRIBIT"],
                "P_CDSITUTRIBITNFE" => $row["CDSITUTRIBITNFE"],
                "P_IDCONSDESONVRNF" => $row["IDCONSDESONVRNF"],
                "P_IDMODBCITNFE" => $row["IDMODBCITNFE"],
                "P_IDMODBCSTITNFE" => $row["IDMODBCSTITNFE"],
                "P_IDMOTDESONERAIT" => $row["IDMOTDESONERAIT"],
                "P_NRITPROIMPOS" => $row["NRITPROIMPOS"],
                "P_NRLANCTONF" =>  $row["NRLANCTONF"],
                "P_NRSEQITPROD" =>  $row["NRSEQITPROD"],
                "P_VRADICALIQFECP" =>  $row["VRADICALIQFECP"],
                "P_VRADICFECP" =>  $row["VRADICFECP"],
                "P_VRALIQITEM" =>  $row["VRALIQITEM"],
                "P_VRBASECALCIT" =>  $row["VRBASECALCIT"],
                "P_VRCONTITIMP" =>  $row["VRCONTITIMP"],
                "P_VRIMPOSTODESON" => $row["VRIMPOSTODESON"],
                "P_VRIMPOSTOIT" =>  $row["VRIMPOSTOIT"],
                "P_VRPARDEDIT" =>  $row["VRPARDEDIT"],
                "P_VRPERMAJBCIT" =>  $row["VRPERMAJBCIT"],
                "P_VRPERREDBCIT" =>  $row["VRPERREDBCIT"]);
                    
        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);

    }
    
    public function deleteImpostoItem($cdfilial, $nrlanctonf, $nrorg, $nritproimpos, $nrseqitprod, $cdimposto) {
        $sql =
            "DELETE
            FROM
                ITEMPROIMPOS
            WHERE
                CDFILIAL = :P_CDFILIAL
                AND NRLANCTONF = :P_NRLANCTONF
                AND NRSEQITPROD = :P_NRSEQITPROD
                AND CDIMPOSTO = :P_CDIMPOSTO
                AND NRITPROIMPOS = :P_NRITPROIMPOS
                AND NRORG = :P_NRORG";
        $params = array(
                    "P_CDFILIAL" => $cdfilial,
                    "P_NRLANCTONF" => $nrlanctonf,
                    "P_NRORG" => $nrorg,
                    "P_NRITPROIMPOS" => $nritproimpos,
                    "P_NRSEQITPROD" => $nrseqitprod,
                    "P_CDIMPOSTO" => $cdimposto
                );
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
    }
    
    public function deleteImpostoNF ($cdfilial, $nrlanctonf, $cdimposto) {
        $sql = 
            "DELETE 
            FROM 
                IMPOSTONF
            WHERE
                NRLANCTONF = :P_NRLANCTONF   AND 
                CDFILIAL = :P_CDFILIAL       AND 
                CDIMPOSTO = :P_CDIMPOSTO";
        
        $params = array(
                    "P_CDFILIAL" => $cdfilial,
                    "P_NRLANCTONF" => $nrlanctonf,
                    "P_CDIMPOSTO" => $cdimposto
                );
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
    }
    public function consultaCDPRODUTO ($cdproduto) {
        $sql = 
            "SELECT CDPRODESTO
                FROM PRODUTO
                    WHERE CDPRODUTO = :P_CDPRODUTO";
        
        $params = array(
                    "P_CDPRODUTO" => $cdproduto
                );
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    public function consultaMaxData ($P_CDPRODUTO, $P_CDFILIEMIT, $P_CDALMOXARIFE, $P_CDLOCALESTOQ, $P_NRLOTEESTQ, $P_NRSUBLOTE, $P_CDPRODESTO, $P_DTPOSICAO) {
        
        $sql = 
            "SELECT MAX(DTPOSIESTQ) DTPOSIESTQ
                FROM POSESTDIA
                    WHERE (CDFILIAL     =   :P_CDFILIEMIT)
                      AND (CDALMOXARIFE =   :P_CDALMOXARIFE)
                      AND (CDLOCALESTOQ =   :P_CDLOCALESTOQ)
                      AND (NRLOTEESTQ   =   :P_NRLOTEESTQ)
                      AND (NRSUBLOTE    =   :P_NRSUBLOTE)
                      AND (CDPRODUTO    =   :P_CDPRODESTO)
                      AND (DTPOSIESTQ  <=   :P_DTPOSICAO)";
        
        $params = array(
                    "P_CDFILIEMIT"      => $P_CDFILIEMIT,
                    "P_CDALMOXARIFE"    => $P_CDALMOXARIFE,
                    "P_CDLOCALESTOQ"    => $P_CDLOCALESTOQ, 
                    "P_NRLOTEESTQ"      => $P_NRLOTEESTQ,
                    "P_NRSUBLOTE"       => $P_NRSUBLOTE,
                    "P_CDPRODESTO"      => $P_CDPRODESTO,
                    "P_DTPOSICAO"       => $P_DTPOSICAO
                );
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    public function VrUnitario ($P_CDPRODUTO, $P_CDFILIEMIT, $P_CDALMOXARIFE, $P_CDLOCALESTOQ, $P_NRLOTEESTQ, $P_NRSUBLOTE, $P_DTPOSICAO, $P_INTPCUSTO, $P_DTPOSIESTQ, $P_QTDE) {
        
        $sql = 
            "SELECT NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIEMIT, D.CDPRODUTO, :P_INTPCUSTO, :P_DTPOSICAO, SYSDATE,
               D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOPROD,
               (D.QTESTOQDIA/P.VRFATOCONV) AS QTESTOQDIA, P.SGUNIDADE,
               DTVALILOTE, P.NMPRODUTO,  'S' IDCNTRESTOQ, D.CDALMOXARIFE,
               D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE, P.VRFATOCONV,
               (QTESTOQANT/P.VRFATOCONV)  AS QTESTOQANT, D.CDPRODUTO, CDPRODESTO
                    FROM PRODUTO P, POSESTDIA D, LOTE L
                        WHERE (D.CDFILIAL       = :P_CDFILIEMIT)
                          AND (D.CDALMOXARIFE   = :P_CDALMOXARIFE)
                          AND (D.CDLOCALESTOQ   = :P_CDLOCALESTOQ)
                          AND (P.CDPRODUTO      = :P_CDPRODUTO)
                          AND (D.NRLOTEESTQ     = :P_NRLOTEESTQ)
                          AND (D.NRSUBLOTE      = :P_NRSUBLOTE)
                          AND (DTPOSIESTQ       = :P_DTPOSIESTQ)
                          AND (D.CDPRODUTO      = P.CDPRODESTO)
                          AND (D.QTESTOQDIA > :P_QTDE OR :P_QTDE = -1) 
                          AND (D.CDFILIAL       = L.CDFILIAL)
                          AND (D.NRLOTEESTQ     = L.NRLOTEESTQ)
                          AND (D.NRSUBLOTE      = L.NRSUBLOTE)
                          AND (D.CDPRODUTO      = L.CDPRODUTO)";
        
        $params = array(
                    "P_CDFILIEMIT"      => $P_CDFILIEMIT,
                    "P_CDALMOXARIFE"    => $P_CDALMOXARIFE,
                    "P_CDLOCALESTOQ"    => $P_CDLOCALESTOQ, 
                    "P_NRLOTEESTQ"      => $P_NRLOTEESTQ,
                    "P_NRSUBLOTE"       => $P_NRSUBLOTE,
                    "P_CDPRODUTO"       => $P_CDPRODUTO,
                    "P_DTPOSICAO"       => $P_DTPOSICAO,
                    "P_INTPCUSTO"       => $P_INTPCUSTO,
                    "P_DTPOSIESTQ"      => $P_DTPOSIESTQ,
                    "P_QTDE"            => $P_QTDE
                );
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    public function insereItemprodutoNFDevolucao($filial, $lancamento, $nrorg) {
        
        $sql = 
            "INSERT INTO ITEMPRODUTO(CDFILIAL, NRLANCTONF, NRORG, NRSEQITPROD, CDPRODUTO, CDPRODEMIS, DSPRODEMIS, QTITEMPROD, VRUNITITEM, VRDESCITEM, VRTOTAITEM, VRCONTITEM, IDCONFORITEM, IDATUVRESTOQ, VRBRUTITEM) 
                SELECT
                    IT.CDFILIAL,
                    IT.NRLANCTONF,
                    IT.NRORG,
                    I.NRSEQITPROD,
                    I.CDPRODUTO,
                    I.CDPRODEMIS,
                    I.DSPRODEMIS,
                    0 AS QTITEMPROD,
                    I.VRUNITITEM,
                    I.VRDESCITEM,
                    I.VRTOTAITEM,
                    I.VRCONTITEM,
                    'S' AS IDCONFORITEM,
                    'S' AS IDATUVRESTOQ,
                    I.VRBRUTITEM
                FROM
                    ITEMPRODUTO    I,
                    ITEMNOTAFISC   IT
                WHERE
                        I.CDFILIAL = IT.CDFILIALIT
                    AND I.NRLANCTONF = IT.NRLANCTONFIT
                    AND IT.NRLANCTONF = :P_NRLANCTONF
                    AND IT.CDFILIAL = :P_CDFILIAL
                    AND I.NRORG = :P_NRORG";
                    
        $params = array(
            'P_CDFILIAL' => $filial,
            'P_NRLANCTONF' => $lancamento,
            'P_NRORG' => $nrorg
        );

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute($params);
        
    }
    
    public function getDataForUnidade($CDFILIAL, $NRORG) {
        #sets parameter data
        $params               = array();
        $params['P_CDFILIAL'] = $CDFILIAL;
        $params['P_NRORG']    = $NRORG;
        //var_dump($params['P_NRORG'], $params['P_CDFILIAL']);
        
        #defines function query
        $sql = "SELECT PARAM.NRORG,
                       PARAM.CDFILIAL,
                       PARAM.IDCTRLESTQ,
                       PARAM.IDESTONLINE,
                       PARAM.IDPERMESTNEG,
                       TPCST.NMTIPOCUSTO               AS NMTIPCUSTEST,
                       PARAM.DTPROCESSA,
                       PARAM.IDINVROTFMES,
                       PARAM.VRPERCINVROT,
                       PARAM.IDORDIMPLANT,
                       PARAM.IDORDAJUINVE,
                       PARAM.IDBLQESTFMES,
                       PARAM.NRDIABLOQEST,
                       PARAM.IDVISPOSAJINVE,
                       NVL(PARAM.IDINVROTDIA,'N')      AS IDINVROTDIA,
                       PARAM.NRDIAFECHMES,
                       PARAM.CDALMOXARIFE,
                       PARAM.CDLOCALESTOQ,
                       PARAM.IDUTCACALMOX,
                       ALMOX.DSALMOXARIFE,
                       LOCAL.DSLOCALESTOQ,
                       NVL(PARAM.IDUTILALMOX, 'N')     AS IDUTILALMOX,
                       NVL(PARAM.IDUTILLCESTQ, 'N')    AS IDUTILLCESTQ,
                       NVL(PARAM.IDUTILLOTE, 'N')      AS IDUTILLOTE,
                       PARAM.DSNUMELOTE,
                       PARAM.DSSEQULOTE,
                       PARAM.IDAJUSIMPLAU,
                       PARAM.IDBLPRCOMEST,
                       PARAM.IDBLOFECHMES,
                       PARAM.IDBLOQFECHNF,
                       PARAM.IDGERARETAJU,
                       PARAM.DSPARFILFIS,
                       SUBSTR(PARAM.DSPARFILFIS, 4, 1) AS IDBLOQESTNEG,
                       PARAM.IDUTCODBARFI,
                       PARAM.IDBLOFERTNEV,
                       PARAM.IDMENSESTNEG,
                       PARAM.IDBLFECHSNFE,
                       PARAM.IDINVROTEST,
                       PARAM.CDTIPCUSTEST,
                       PARAM.IDBLOQFECDIA,
                       PARAM.DSORDENALOTE,
                       PARAM.IDPERPROINAT,
                       PARAM.IDINVEROTREQ,
                       PARAM.IDENTRSEQLOTE,
                       PARAM.IDENTRAUTLOTE,
                       PARAM.IDCONSDIAUTFEC,
                       PARAM.IDFDIAENTREST,
                       PARAM.IDFDIASAIDEST,
                       PARAM.IDFDIACARDAP,
                       PARAM.IDFDIARETIEST,
                       PARAM.IDFDIAREQEST,
                       PARAM.IDFDIAREQPLA,
                       PARAM.IDFDIAEFEPLA,
                       PARAM.IDFDIAEFEFAT,
                       PARAM.VRPETOLEINVROT,
                       PARAM.IDREQCOMPEST,
                       PARAM.CDGRUPPRMOVEST,
                       PARAM.CDGRUPPRCTRLTEM,
                       PARAM.IDINVESTCEGO,
                       PARAM.CDSERVBAIESTCEG,
                       PARAM.CDTPRETBXESTCEG,
                       PARAM.IDLANCDTMENFIL,
                       TRUNC(PARAM.NRDIASENTRFIL)      AS NRDIASENTRFIL,
                       TRUNC(PARAM.NRDIASSAIDFIL)      AS NRDIASSAIDFIL,
                       TRUNC(PARAM.NRDIASAJUSFIL)      AS NRDIASAJUSFIL,
                       PARAM.IDLANCDTMAIFIL,
                       TRUNC(PARAM.NRDIASENTRPDFIL)    AS NRDIASENTRPDFIL,
                       TRUNC(PARAM.NRDIASSAIDPDFIL)    AS NRDIASSAIDPDFIL,
                       TRUNC(PARAM.NRDIASAJUSPDFIL)    AS NRDIASAJUSPDFIL,
                       NVL(PARAM.IDUTMODPRODFIL, 'N')  AS IDUTMODPRODFIL,
                       NVL(PARAM.IDPERMVISPOSEST, 'S') AS IDPERMVISPOSEST,
                       PARAM.IDOBRIGAAJUINVE,
                       NVL(PARAM.IDCALCRECBRUTA, 'N')  AS IDCALCRECBRUTA,
                       NVL(PARAM.IDOBREALACEDIV, 'N')  AS IDOBREALACEDIV,
                       NVL(PARAM.IDBLOQFECINVPE, 'N')  AS IDBLOQFECINVPE,
                       PARAM.IDBLOQEFEPDIFEFEF,
                       PARAM.IDBLOQAJUSNEVOL,
                       NVL(PARAM.IDETIQPRODOL, 'N')    AS IDETIQPRODOL,
                       NVL(PARAM.IDENTRUNILOTE, 'N')   AS IDENTRUNILOTE,
                       NVL(PARAM.IDCTRLDISTRIB, 'N')   AS IDCTRLDISTRIB,
                       PARGE.CDPARGERAL,
                       PARGE.VRPECURVAA,
                       PARGE.VRPECURVAB,
                       PARGE.VRPECURVAC,
                       SERV.NMSERVICO,
                       TPRETI.NMTIPORETI
                  FROM PARAMFILIAL PARAM
             LEFT JOIN ALMOXARIFADO ALMOX
                    ON PARAM.CDFILIAL     = ALMOX.CDFILIAL
                   AND PARAM.CDALMOXARIFE = ALMOX.CDALMOXARIFE
             LEFT JOIN LOCALESTOQUE LOCAL
                    ON PARAM.CDFILIAL     = LOCAL.CDFILIAL
                   AND PARAM.CDALMOXARIFE = LOCAL.CDALMOXARIFE
                   AND PARAM.CDLOCALESTOQ = LOCAL.CDLOCALESTOQ
             LEFT JOIN TIPOCUSTO    TPCST
                    ON PARAM.CDTIPCUSTEST = TPCST.CDTIPOCUSTO
             LEFT JOIN PARAMGERAL   PARGE
                    ON 1 = 1
             LEFT JOIN SERVICO      SERV
                    ON PARAM.CDSERVBAIESTCEG = SERV.CDSERVFILI
                    AND :P_CDFILIAL = SERV.CDFILIAL
             LEFT JOIN TIPORETI     TPRETI
                    ON PARAM.CDTPRETBXESTCEG = TPRETI.CDTIPORETI
                 WHERE PARAM.CDFILIAL     = :P_CDFILIAL
                   AND PARAM.NRORG        = :P_NRORG";

        #prepares query and executes it with parameters
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }
    
    public function getDataForGeral($NRORG) {
        #sets parameter data
        $params               = array();
        $params['P_NRORG']    = $NRORG;

        #defines function query
        $sql = "SELECT CDPARESTOQ,
                       PE.NRORG,
                       VRPETOLEENTR,
                       IDTRATAVENC,
                       IDLANCDTMEN,
                       TRUNC(NRDIASENTR)          AS NRDIASENTR,
                       TRUNC(NRDIASSAID)          AS NRDIASSAID,
                       TRUNC(NRDIASAJUS)          AS NRDIASAJUS,
                       IDLANCDTMAI,
                       IDCONSFORNFJ,
                       IDBLOQFORIRR,
                       CDGROPERRETI,
                       CDGROPERSOLI,
                       TRUNC(NRMAXITROMA)         AS NRMAXITROMA,
                       CDGROPERFMES,
                       TRUNC(NRDIASENTRPD)        AS NRDIASENTRPD,
                       TRUNC(NRDIASSAIDPD)        AS NRDIASSAIDPD,
                       TRUNC(NRDIASAJUSPD)        AS NRDIASAJUSPD,
                       IDCONAUTIRRG,
                       IDBLOQPRESTO,
                       IDALTDTFECDIA,
                       IDGTGTXTFDIA,
                       CDGROPERREQ,
                       IDCTRLESTNVPROD,
                       NRDIASMOVESTCEG,
                       CDGROPERLOT,
                       CDGROPERVISU,
                       NVL(IDOBFABRICLANCNF, 'N') AS IDOBFABRICLANCNF,
                       NVL(IDCONSMARCPRFAB, 'N')  AS IDCONSMARCPRFAB,
                       TRUNC(NRMESANTEMOVIEST) AS NRMESANTEMOVIEST,
                       DTMOVIESTINI,
                       DTMOVIESTFIN,
                       HRAGEMOVIEST,
                       IDPERMEXCITINVE,
                       PG.IDUTILDTPROC,
                       CDGROPERATUEST
                  FROM PARAMESTOQUE PE
                  LEFT JOIN PARAMGERAL PG
                  ON 1 = 1
                 WHERE PE.NRORG = :P_NRORG";

        #prepares query and executes it with parameters
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        #fetch all result rows and returns
        return $std->fetchAll();
    }
    
    public function newParamestoque(){
        $sql = "INSERT INTO PARAMESTOQUE(CDPARESTOQ) (SELECT '1' FROM DUAL WHERE 0 = (SELECT COUNT(*) FROM PARAMESTOQUE))";

        #prepares query and executes it with parameters
        $std = $this->connection->prepare($sql);
        $std->execute();
    }
}