<?php

namespace Teknisa\DF\Factories;

use Teknisa\Model\Fiscal\Entities\Trabreplicdoc;

use Geral\Util\DateUtil;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class ReplicacaoDocumentosFactory {

    private $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }

    // Formata a data de forma que possa ser inserida no banco
    public static function formataData($data) {
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);

        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);
        return $data;
    }
    
    public function getDocumentos($filial, $operacao, $tnrreplicdoc, $nrreplicdoc)
    {
        $sql = <<<SQL
        SELECT CDFILIAL,    R.IDENTRSAIDOP, R.CDTIPOOPER,
                NRREPLICDOC, DSREPLICDOC,    IDREPLAUTONF
         FROM REPLICDOC R
         WHERE (CDFILIAL     = :P_CDFILIEMIT)    AND
               (CDTIPOOPER   = :P_CDTIPOOPER)  AND
               (IDENTRSAIDOP = 'S')            AND
               ((:P_TNRREPLICDOC = 'A' AND IDREPLAUTONF = 'S') OR
               ((NRREPLICDOC = :P_NRREPLICDOC) OR (:P_TNRREPLICDOC = 'T')))
         ORDER BY NRREPLICDOC, R.IDENTRSAIDOP, R.CDTIPOOPER
SQL;
        $params = array(
            "P_CDFILIEMIT" => $filial,
            "P_CDTIPOOPER" => $operacao,
            "P_TNRREPLICDOC" => $tnrreplicdoc,
            "P_NRREPLICDOC" => $nrreplicdoc
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getDocumentosTabelaTemporaria($filial, $lancamento, $filialOrigem, $filialDestino, $entradaSaida, $operacao, $almoxarifado, $localEstoque)
    {
        $sql = <<<SQL
        SELECT CDFILIALTRD, NRLANCTONFTRD
          FROM TRABREPLICDOC
         WHERE CDFILIALTRD   = :P_CDFILIAL
           AND NRLANCTONFTRD = :P_NRLANCTONF
           AND CDFILIORIGTRD = :P_CDFILIORIGTRD
           AND CDFILIDESTTRD = :P_CDFILIDESTTRD
           AND IDENTRSAIDTRD = :P_IDENTRSAIDTRD
           AND CDTIPOOPERTRD = :P_CDTIPOOPERTRD
           AND CDALMOXARTRD  = :P_CDALMOXARTRD
           AND CDLOCESTQTRD  = :P_CDLOCESTQTRD
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRLANCTONF" => $lancamento,
            "P_CDFILIORIGTRD" => $filialOrigem,
            "P_CDFILIDESTTRD" => $filialDestino,
            "P_IDENTRSAIDTRD" => $entradaSaida,
            "P_CDTIPOOPERTRD" => $operacao,
            "P_CDALMOXARTRD" => $almoxarifado,
            "P_CDLOCESTQTRD" => $localEstoque
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getProdutoSegmento($produto, $segmento)
    {
        $sql = <<<SQL
        SELECT
              P.CDPRODUTO, P.CDARVPROD,
              P.NMPRODUTO, S.IDPROSEGATIV, P.VRFATOCONV
           FROM
              PRODUTO P, PRODUTOSEG S
           WHERE
              P.CDPRODUTO = S.CDPRODUTO  AND
              S.CDSEGMER  = :P_SEGMENTO  AND
              P.CDARVPROD = :P_PRODUTO
SQL;
        $params = array(
            "P_SEGMENTO" => $segmento,
            "P_PRODUTO" => $produto
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getProdutoSegmentoModuloFilial($produto, $segmento, $filial)
    {
        $sql = <<<SQL
        SELECT
              P.CDPRODUTO, P.CDARVPROD,
              P.NMPRODUTO, S.IDPROSEGATIV, P.VRFATOCONV
           FROM
              PRODUTO P, PRODUTOSEG S, PRODMODFILI
           WHERE
              P.CDPRODUTO = S.CDPRODUTO  AND
              S.CDSEGMER  = :P_SEGMENTO  AND
              P.CDARVPROD = :P_PRODUTO AND
             PRODMODFILI.CDFILIAL  = :P_FILIAL AND
             PRODMODFILI.CDPRODUTO = P.CDPRODUTO
SQL;
        $params = array(
            "P_SEGMENTO" => $segmento,
            "P_PRODUTO" => $produto,
            "P_FILIAL" => $filial
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getItensDocumento($filial, $numeroReplicacao)
    {
        $sql = <<<SQL
        SELECT
           I.CDFILIAL,
           I.NRREPLICDOC,
           I.CDFILIORIGRD,
           I.CDFILIDESTRD,
           DECODE(I.IDENTRSAIDOP, 'E', I.CDALMODESTRD, I.CDALMOORIGRD) AS CDALMOXARIFE,
           DECODE(I.IDENTRSAIDOP, 'E', I.CDLESTDESTRD, I.CDLESTORIGRD) AS CDLOCALESTOQ,
           I.IDENTRSAIDOP,
           I.CDTIPOOPER,
           I.DSITREPLDOC,
           I.CDSERIE
        FROM
           ITREPLICDOC I
        WHERE
           (I.CDFILIAL = :P_CDFILIEMIT) AND
           (I.NRREPLICDOC = :P_NRREPLICDOC)
SQL;
        $params = array(
            "P_CDFILIEMIT" => $filial,
            "P_NRREPLICDOC" => $numeroReplicacao
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function produtoInativo($filial, $lancamento, $filialDestino)
    {
        $sql = <<<SQL
        SELECT N.CDFILIAL, N.NRLANCTONF, N.CDFILIRECE, I.CDPRODUTO,
             P.IDPRODATIV, T.CDARVPROD, T.NMPRODUTO
          FROM NOTAFISCAL N, ITEMPRODUTO I, PRODFILI P, PRODUTO T
          WHERE N.CDFILIAL   = :P_CDFILIAL
            AND N.NRLANCTONF = :P_NRLANCTONF
            AND P.CDFILIAL   = :P_CDFILIRECE
            AND N.CDFILIAL   = I.CDFILIAL
            AND N.NRLANCTONF = I.NRLANCTONF
            AND N.CDFILIRECE = P.CDFILIAL
            AND I.CDPRODUTO  = P.CDPRODUTO
            AND ((P.IDPRODATIV = 'I') OR (T.IDPRODATIVPR = 'N'))
            AND I.CDPRODUTO  = T.CDPRODUTO
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRLANCTONF" => $lancamento,
            "P_CDFILIRECE" => $filialDestino
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function produtoSegmento($filial, $lancamento)
    {
        $sql = <<<SQL
        SELECT N.CDFILIRECE AS FILIAL, F.CDSEGMER AS SEGMENTO, PF.IDUTMODPRODFIL AS MODPRODFIL, T.CDARVPROD AS PRODUTO
           FROM NOTAFISCAL N, ITEMPRODUTO I, PRODUTO T, FILIAL F, PARAMFILIAL PF
          WHERE N.CDFILIAL    = :P_CDFILIAL
            AND N.NRLANCTONF  = :P_NRLANCTONF
            AND N.CDFILIAL    = I.CDFILIAL
            AND N.NRLANCTONF  = I.NRLANCTONF
            AND I.CDPRODUTO   = T.CDPRODUTO
            AND F.CDFILIAL    = N.CDFILIRECE
            AND F.CDFILIAL    = PF.CDFILIAL
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRLANCTONF" => $lancamento
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function parametrosDFS($filial)
    {
        $sql = <<<SQL
        SELECT PF.IDLANCDTMAIFIL, PF.IDLANCDTMENFIL, PF.NRDIASSAIDFIL, PF.NRDIASSAIDPDFIL,
             PE.IDLANCDTMAI, PE.IDLANCDTMEN, PE.NRDIASSAID, PE.NRDIASSAIDPD
          FROM (SELECT IDLANCDTMAIFIL, IDLANCDTMENFIL, NRDIASSAIDFIL, NRDIASSAIDPDFIL
                  FROM PARAMFILIAL
                 WHERE CDFILIAL = :P_CDFILIAL) PF,
               (SELECT IDLANCDTMAI, IDLANCDTMEN, NRDIASSAID, NRDIASSAIDPD
                  FROM PARAMESTOQUE) PE
SQL;
        $params = array(
            "P_CDFILIAL" => $filial
        );

        return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
    }
    
    public function getNotasOrigemReplicacao($iddocreplic)
    {
        $sql = <<<SQL
        SELECT N.CDFILIAL, N.NRORG, N.NRLANCTONF, CDFILIEMIT, CDFILIRECE,
                N.IDENTRSAIDOP, N.CDTIPOOPER, N.CDSERIE,
                ' ' AS DSITREPLDOC, T.NMTIPOOPER,
                D.NMFILIAL AS NMFILIDEST,
                O.NMFILIAL AS NMFILIORIG,
                ' ' AS NRREPLICDOC,
                IDDOCREPLIC, N.IDSTATUS,
                ' ' AS ALMOXARIFE, ' ' AS LOCLAIZACAO,
                ' ' AS DSALMOXARIFE, ' ' AS DSLOCALESTOQ, N.CDTIPOOPER||'  '||T.NMTIPOOPER AS TIPO,
                NRLANCESTQ
           FROM NOTAFISCAL N, FILIAL D, FILIAL O, TIPOOPERAC T, LANCTOESTOQ L
           WHERE IDDOCREPLIC    = :P_IDDOCREPLIC AND
                 CDFILIEMIT     = O.CDFILIAL                    AND
                 CDFILIRECE     = D.CDFILIAL(+)                    AND
                 N.CDFILIAL     = L.CDFILIAL(+)                 AND
                 N.NRLANCTONF   = L.NRLANCTONF(+)               AND
                 N.IDENTRSAIDOP = 'S'                           AND
                 T.IDENTRSAIDOP = N.IDENTRSAIDOP                AND
                 T.CDTIPOOPER   = N.CDTIPOOPER                  AND
                 N.IDBLOQUEADO  = 'N'
           UNION
           SELECT N.CDFILIAL, N.NRORG, N.NRLANCTONF, CDFILIEMIT, CDFILIRECE,
                  N.IDENTRSAIDOP, N.CDTIPOOPER, CDSERIEENTR,
                  ' ' AS DSITREPLDOC, T.NMTIPOOPER,
                  D.NMFILIAL AS NMFILIDEST,
                  O.NMFILIAL AS NMFILIORIG,
                  ' ' AS NRREPLICDOC,
                  IDDOCREPLIC, N.IDSTATUS,
                  ' ' AS ALMOXARIFE, ' ' AS LOCLAIZACAO,
                  ' ' AS DSALMOXARIFE, ' ' AS DSLOCALESTOQ, N.CDTIPOOPER||'  '||T.NMTIPOOPER AS TIPO,
                  NRLANCESTQ
           FROM NOTAFISCAL N, FILIAL D, FILIAL O, TIPOOPERAC T, LANCTOESTOQ L
           WHERE IDDOCREPLIC    = :P_IDDOCREPLIC AND
                 CDFILIEMIT     = O.CDFILIAL                    AND
                 CDFILIRECE     = D.CDFILIAL(+)                 AND
                 N.IDENTRSAIDOP = 'E'                           AND
                 N.CDFILIAL     = L.CDFILIAL(+)                 AND
                 N.NRLANCTONF   = L.NRLANCTONF(+)               AND
                 T.IDENTRSAIDOP = N.IDENTRSAIDOP                AND
                 T.CDTIPOOPER   = N.CDTIPOOPER                  AND
                 N.IDBLOQUEADO  = 'N'
           ORDER BY 3, 4, 5, 6
SQL;
        $params = array(
            "P_IDDOCREPLIC" => $iddocreplic
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function getNotafiscal($filial, $lancamento)
    {
        $sql = <<<SQL
        SELECT
             A.CDFILIAL     ,A.DTENTRSAID   ,A.CDSERIE      ,
             A.NRLANCTONF   ,A.VRPRODUTOS   ,A.IDTPIJURTRAN ,
             A.CDEMPRESA    ,A.VRNOTAFISC   ,A.NRINSJURTRAN ,
             A.CDINSCESTA   ,A.NRPLACAVEIC  ,A.CDFISOPENTF  ,
             A.CDINSCMUNI   ,A.SGUFVEIC     ,A.DSCDFISOPENF ,
             A.CDFILIRECE   ,A.IDPGTOFRETE  ,A.IDDOCCONTAB  ,
             A.IDTPIJURNF   ,A.DSQTDEVOLU   ,A.IDSTATUS     ,
             NVL(D.NRINSJURORIG, A.NRINSJURNF) NRINSJURNF,
             A.DSESPECVOLU  ,A.VRPBRUTOVOLU ,A.DSMARCAVOLU  ,
             A.VRPLIQUVOLU  ,A.CDFORNECED   ,A.DSNUMEVOLU   ,
             A.IDENTRSAIDOP ,A.IDDOCFINANC  ,A.IDBLOQUEADO  ,
             A.NRNOTAFISC   ,A.CDTIPOOPER   ,A.IDDOCFISCAL  ,
             A.DTEMISSAO    ,A.IDERROS      ,A.IDAVISOS     ,
             A.CDMODDOCFISC ,A.IDSISTEMA    ,A.CDFILIEMIT   ,
             A.CDCLIENTE    ,A.IDORIGDESTNF ,A.CDINTEGRACAO ,
             A.NRLICENCAUSO ,A.IDDOCFATUR   ,A.DTLANCAMENNF ,
             T.IDGERAVENC   ,T.IDUTILVENC   ,A.DTCOMPFATUR  ,
             DF.DSMODDOCFISC,A.DTINIREFEFAT ,A.DTFINREFEFAT ,
             NRCPFCNPJDFS   ,A.IDSTATUSNFE  ,A.IDLANCSERVICO, A.CDCFOPINFUSU, A.DSCFOPINFUSU,
             A.IDDOCREPLIC  ,A.NRINSJURNFPAULI, A.IDPRESENCIAL
          FROM NOTAFISCAL A, TIPOOPERAC T, MODDOCFISC DF, DADOSORIGNF D
          WHERE A.CDFILIAL     = :P_CDFILIAL     AND
                A.NRLANCTONF   = :P_NRLANCTONF   AND
                A.IDENTRSAIDOP = T.IDENTRSAIDOP  AND
                A.CDTIPOOPER   = T.CDTIPOOPER    AND
                A.CDFILIAL     = D.CDFILIAL(+)   AND
                A.NRLANCTONF   = D.NRLANCTONF(+) AND
                ((A.IDGEROUTDOCU IS NULL) OR (A.IDGEROUTDOCU = 'A')) AND
                DF.CDMODDOCFISC(+) = A.CDMODDOCFISC
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRLANCTONF" => $lancamento
        );

        return $this->entityManager->getConnection()->fetchAssoc($sql, $params);
    }
    
    public function verificarReplicacao($iddocreplic)
    {
        $sql = <<<SQL
        SELECT N.CDFILIAL, N.NRLANCTONF, N.IDENTRSAIDOP, N.VRNOTAFISC,
                 N.CDFILIEMIT, N.CDFILIRECE, L.NRLANCESTQ, N.CDTIPOOPER,
                 N.DTENTRSAID
          FROM NOTAFISCAL N, LANCTOESTOQ  L
          WHERE (N.IDDOCREPLIC = :P_IDDOCREPLIC) AND
                (N.CDFILIAL    = L.CDFILIAL)     AND
                (N.NRLANCTONF  = L.NRLANCTONF)
SQL;
        $params = array(
            "P_IDDOCREPLIC" => $iddocreplic
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function verificarReplicacaoEstoque($filial, $lancamento, $filialOrigem)
    {
        $sql = <<<SQL
        SELECT N.CDFILIAL, N.NRLANCTONF, N.IDENTRSAIDOP,
              I.CDALMOXARIFE, I.CDLOCALESTOQ, I.NRLOTEESTQ,
              P.IDUTILALMOX, P.IDUTILLCESTQ, P.IDUTILLOTE, NVL(PF.IDCNTRESTOQ, 'S') AS IDCNTRESTOQ
         FROM NOTAFISCAL N, ITEMPRODUTO I, PARAMFILIAL P, PRODFILI PF
        WHERE N.CDFILIAL   = :P_CDFILIAL
          AND N.NRLANCTONF = :P_NRLANCTONF
          AND N.CDFILIAL   = I.CDFILIAL
          AND N.NRLANCTONF = I.NRLANCTONF
          AND (CASE WHEN N.IDENTRSAIDOP = 'S' THEN N.CDFILIEMIT ELSE N.CDFILIRECE END) = :P_CDFILIORIG
          AND :P_CDFILIORIG = P.CDFILIAL
          AND PF.CDFILIAL   = P.CDFILIAL
          AND PF.CDPRODUTO  = I.CDPRODUTO
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRLANCTONF" => $lancamento,
            "P_CDFILIORIG" => $filialOrigem
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function verificarFilialUtilizaOrcamentoGOP($filial)
    {
        $sql = <<<SQL
        SELECT NVL(IDCONSORCFIL,'N') IDCONSORCFIL, VRPEORCBLOQFIL, VRPEORCALERTFIL
        FROM FILIAL
       WHERE CDFILIAL = :P_CDFILIAL
         AND NVL(IDCNTRRESC,'N') = 'N'
SQL;
        $params = array(
            "P_CDFILIAL" => $filial
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function buscarMaiorVersaoOrcamento($ano)
    {
        $sql = <<<SQL
        SELECT TO_CHAR(MAX(CDVERSAOORCA)) CDVERSAOORCA
           FROM VERSAOORCA
          WHERE CDANOVERORCA = :P_CDANOVERORCA
SQL;
        $params = array(
            "P_CDANOVERORCA" => $ano
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function buscarOrcamentoConsumo($filial, $ano, $mes, $versao)
    {
        $sql = <<<SQL
        SELECT SUM(NVL(DECODE(:P_MES,'01', VRORCAMENTOJAN,
                DECODE(:P_MES,'02', VRORCAMENTOFEV,
                DECODE(:P_MES,'03', VRORCAMENTOMAR,
                DECODE(:P_MES,'04', VRORCAMENTOABR,
                DECODE(:P_MES,'05', VRORCAMENTOMAI,
                DECODE(:P_MES,'06', VRORCAMENTOJUN,
                DECODE(:P_MES,'07', VRORCAMENTOJUL,
                DECODE(:P_MES,'08', VRORCAMENTOAGO,
                DECODE(:P_MES,'09', VRORCAMENTOSET,
                DECODE(:P_MES,'10', VRORCAMENTOOUT,
                DECODE(:P_MES,'11', VRORCAMENTONOV, VRORCAMENTODEZ))))))))))),0)) VRORCAMENTOC

           FROM ITEMORCAOP
          WHERE CDFILIAL     = :P_CDFILIAL
            AND NRANOORCAOP  = :P_NRANOORCAOP
            AND CDVERSORCAOP = :P_CDVERSORCAOP
            AND IDTIPOORCAOP = 'C'
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRANOORCAOP" => $ano,
            "P_MES" => str_pad($mes, 2, "0", STR_PAD_LEFT),
            "P_CDVERSORCAOP" => $versao
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function buscarOrcamentoDespesa($filial, $ano, $mes, $versao, $operacao)
    {
        $sql = <<<SQL
        SELECT SUM(NVL(DECODE(:P_MES,'01', VRORCAMENTOJAN,
                DECODE(:P_MES,'02', VRORCAMENTOFEV,
                DECODE(:P_MES,'03', VRORCAMENTOMAR,
                DECODE(:P_MES,'04', VRORCAMENTOABR,
                DECODE(:P_MES,'05', VRORCAMENTOMAI,
                DECODE(:P_MES,'06', VRORCAMENTOJUN,
                DECODE(:P_MES,'07', VRORCAMENTOJUL,
                DECODE(:P_MES,'08', VRORCAMENTOAGO,
                DECODE(:P_MES,'09', VRORCAMENTOSET,
                DECODE(:P_MES,'10', VRORCAMENTOOUT,
                DECODE(:P_MES,'11', VRORCAMENTONOV, VRORCAMENTODEZ))))))))))),0)) VRORCAMENTOE

           FROM ITEMORCAOP
          WHERE CDFILIAL     = :P_CDFILIAL
            AND NRANOORCAOP  = :P_NRANOORCAOP
            AND CDVERSORCAOP = :P_CDVERSORCAOP
            AND IDTIPOORCAOP = 'E'
            AND CDTIPOOPER   = :P_CDTIPOOPER
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRANOORCAOP" => $ano,
            "P_MES" => str_pad($mes, 2, "0", STR_PAD_LEFT),
            "P_CDVERSORCAOP" => $versao,
            "P_CDTIPOOPER" => $operacao
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function buscarConsumo($filial, $lancamento, $filirece, $controlaEstoque)
    {
        $sql = <<<SQL
        SELECT DISTINCT NVL(IDCNTRESTOQ,'S') IDCNTRESTOQ
			  FROM ITEMPRODUTO IP, PRODFILI PF, PRODUTO P
			 WHERE IP.CDFILIAL = :P_CDFILIAL
			   AND IP.NRLANCTONF = :P_NRLANCTONF
			   AND PF.CDFILIAL = :P_CDFILIRECE
			   AND IP.CDPRODUTO = P.CDPRODUTO
			   AND PF.CDPRODUTO = P.CDPRODESTO
			   AND NVL(PF.IDCNTRESTOQ,'S') = :P_IDCNTRESTOQ
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_NRLANCTONF" => $lancamento,
            "P_CDFILIRECE" => $filirece,
            "P_IDCNTRESTOQ" => $controlaEstoque
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function verificarEntradas($filial, $dataInicial, $dataFinal)
    {
        $sql = <<<SQL
        SELECT SUM(VRTOTLANCTO) VRTOTALENTR
        FROM LANCTOESTOQ L, ITLANCTOEST I
       WHERE L.CDFILIMOVI = :P_CDFILIRECE
         AND L.DTLANCESTQ BETWEEN :P_DTINICIAL AND :P_DTFINAL
         AND L.CDFILIAL   = I.CDFILIAL
         AND L.NRLANCESTQ = I.NRLANCESTQ
         AND L.IDTPLANCTO IN ('2','E')
         AND NVL(I.IDEVOLESTOQ,'S') = 'S'
SQL;
        $params = array(
            "P_CDFILIRECE" => $filial,
            "P_DTINICIAL" => $dataInicial,
            "P_DTFINAL" => $dataFinal
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function verificarDespesas($filial, $dataInicial, $dataFinal, $operacao)
    {
        $sql = <<<SQL
        SELECT SUM(VRTOTAITEM) VRDESPESA
           FROM NOTAFISCAL N, ITEMPRODUTO I, TIPOOPERACROP T, PRODFILI PF, PRODUTO P,
           (
           SELECT DISTINCT A.CDPRODUTO
           FROM PRODUTOROP R, PRODUTO A, PRODUTO B
           WHERE (INSTR(A.CDARVPROD, B.CDARVPROD) = 1)
             AND R.CDPRODUTO = B.CDPRODUTO
             AND R.CDFILIAL = TIPOPARAM_DRO(:P_CDFILIAL)
             AND IDTPPRODUTOROP = 'DE'
    
           ) A
           WHERE  N.CDFILIRECE   = :P_CDFILIAL
              AND N.DTENTRSAID BETWEEN :P_DTINICIAL AND :P_DTFINAL
              AND N.IDENTRSAIDOP = 'E'
              AND I.CDFILIAL     = N.CDFILIAL
              AND I.NRLANCTONF   = N.NRLANCTONF
              AND N.CDTIPOOPER   = T.CDTIPOOPER
              AND N.IDENTRSAIDOP = T.IDENTRSAIDOP
              AND T.CDFILIAL     = TIPOPARAM_DRO(:P_CDFILIAL)
              AND T.IDTPOPERACROP = 'DE'
              AND P.CDPRODUTO = I.CDPRODUTO
              AND I.CDPRODUTO = A.CDPRODUTO
              AND PF.CDFILIAL = N.CDFILIRECE
              AND PF.CDPRODUTO = P.CDPRODESTO
              AND NVL(PF.IDCNTRESTOQ,'S')   = 'N'
              AND T.CDTIPOOPER   = :P_CDTIPOOPER
SQL;
        $params = array(
            "P_CDFILIAL" => $filial,
            "P_DTINICIAL" => $dataInicial,
            "P_DTFINAL" => $dataFinal,
            "P_CDTIPOOPER" => $operacao
        );

        return $this->entityManager->getConnection()->fetchAll($sql, $params);
    }
    
    public function inserirReplicacaoTemporaria(
        $filial, $lancamento, $filialOrigem, $filialDestino, $entradaSaida,
        $operacao, $serie, $almoxarifado, $localEstoque, $organizacao, $operador
    ) {
        try {
            $data = DateUtil::getDataAtual();
            $trabreplicdoc = new Trabreplicdoc();
            $trabreplicdoc->setCdfilialtrd($filial);
            $trabreplicdoc->setNrlanctonftrd($lancamento);
            $trabreplicdoc->setCdfiliorigtrd($filialOrigem);
            $trabreplicdoc->setCdfilidesttrd($filialDestino);
            $trabreplicdoc->setIdentrsaidtrd($entradaSaida);
            $trabreplicdoc->setCdtipoopertrd($operacao);
            $trabreplicdoc->setCdserietrd($serie);
            $trabreplicdoc->setCdalmoxartrd($almoxarifado);
            $trabreplicdoc->setCdlocestqtrd($localEstoque);

            $trabreplicdoc->setNrorg($organizacao);
            $trabreplicdoc->setNrorginclusao($organizacao);
            $trabreplicdoc->setNrorgultatu($organizacao);
            $trabreplicdoc->setCdoperinclusao($operador);
            $trabreplicdoc->setCdoperultatu($operador);
            $trabreplicdoc->setDtinclusao($data);
            $trabreplicdoc->setDtultatu($data);
            
            $this->entityManager->persist($trabreplicdoc);
            $this->entityManager->flush();

            return $trabreplicdoc;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function deletarTodaReplicacaoTemporaria(){
        try {
            $sql = <<<SQL
        DELETE FROM TRABREPLICDOC
SQL;

        $std = $this->entityManager->getConnection()->prepare($sql);
        return $std->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
